# -*- coding: utf-8 -*-

import sre_constants

import regex as re

from prettyqt import gui, core


class RegexPatternValidator(gui.Validator):
    error_occured = core.Signal(str)
    pattern_updated = core.Signal(object)

    def __repr__(self):
        return f"{type(self).__name__}()"

    def __eq__(self, other: object):
        return isinstance(other, type(self))

    def validate(self, text: str, pos: int = 0) -> tuple:
        # if text == "":
        #     self.compiled = None
        #     return (self.Intermediate, text, pos)
        try:
            compiled = re.compile(text)
        except sre_constants.error as e:
            self.error_occured.emit(str(e))
            self.pattern_updated.emit(None)
            return (self.Intermediate, text, pos)
        except re._regex_core.error as e:
            self.error_occured.emit(str(e))
            self.pattern_updated.emit(None)
            return (self.Intermediate, text, pos)
        else:
            self.error_occured.emit("")
            self.pattern_updated.emit(compiled)
            return (self.Acceptable, text, pos)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    w = widgets.LineEdit()
    val = RegexPatternValidator()
    w.set_validator(val)
    w.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets, QtCore

from prettyqt import gui
from prettyqt.utils import bidict, InvalidParamError

STATES = bidict(
    unchecked=QtCore.Qt.Unchecked,
    partial=QtCore.Qt.PartiallyChecked,
    checked=QtCore.Qt.Checked,
)


class TableWidgetItem(QtWidgets.QTableWidgetItem):
    def __setitem__(self, index: int, value):
        self.setData(index, value)

    def __getitem__(self, index: int):
        return self.data(index)

    def serialize_fields(self):
        return dict(
            text=self.text(),
            tool_tip=self.toolTip(),
            status_tip=self.statusTip(),
            checkstate=self.get_checkstate(),
            icon=gui.Icon(self.icon()) if not self.icon().isNull() else None,
            data=self.data(QtCore.Qt.UserRole),
        )

    def set_icon(self, icon: gui.icon.IconType):
        """Set the icon for the action.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(icon)

    def set_checkstate(self, state: str):
        """Set checkstate of the checkbox.

        valid values are: unchecked, partial, checked

        Args:
            state: checkstate to use

        Raises:
            InvalidParamError: invalid checkstate
        """
        if state not in STATES:
            raise InvalidParamError(state, STATES)
        self.setCheckState(STATES[state])

    def get_checkstate(self) -> str:
        """Return checkstate.

        possible values are "unchecked", "partial", "checked"

        Returns:
            checkstate
        """
        return STATES.inv[self.checkState()]

    def get_background(self) -> gui.Brush:
        return gui.Brush(self.background())

    def get_foreground(self) -> gui.Brush:
        return gui.Brush(self.foreground())

    def get_font(self) -> gui.Font:
        return gui.Font(self.font())

    def get_icon(self) -> gui.Icon:
        return gui.Icon(self.icon())


if __name__ == "__main__":
    item = TableWidgetItem()



# -*- coding: utf-8 -*-

from typing import List

from qtpy import QtLocation, PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning


from prettyqt import location, positioning
from prettyqt.utils import bidict, InvalidParamError

RELEVANCE_HINT = bidict(
    unspecified=QtLocation.QPlaceSearchRequest.UnspecifiedHint,
    distance=QtLocation.QPlaceSearchRequest.DistanceHint,
    lexical_place_name=QtLocation.QPlaceSearchRequest.LexicalPlaceNameHint,
)

VISIBILITY = bidict(
    unspecified=QtLocation.QLocation.UnspecifiedVisibility,
    device=QtLocation.QLocation.DeviceVisibility,
    private=QtLocation.QLocation.PrivateVisibility,
    public=QtLocation.QLocation.PublicVisibility,
)


class PlaceSearchRequest(QtLocation.QPlaceSearchRequest):
    def get_visibility_scope(self) -> str:
        """Return the scope of the visibility.

        Possible values are "unspecified", "device", "private", "public"

        Returns:
            Visibility scope
        """
        return VISIBILITY.inv[self.visibilityScope()]

    def get_categories(self) -> List[location.PlaceCategory]:
        return [location.PlaceCategory(i) for i in self.categories()]

    def get_search_area(self) -> positioning.GeoShape:
        area = self.searchArea()
        if isinstance(area, QtPositioning.QGeoCircle):
            return positioning.GeoCircle(area)
        elif isinstance(area, QtPositioning.QGeoPath):
            return positioning.GeoPath(area)
        elif isinstance(area, QtPositioning.QGeoPolygon):
            return positioning.GeoPolygon(area)
        elif isinstance(area, QtPositioning.QGeoRectangle):
            return positioning.GeoRectangle(area)
        else:
            return positioning.GeoShape(area)

    def set_relevance_hint(self, hint: str):
        """Set the relevance hint.

        Allowed values are "unspecified", "distance", "lexical_place_name"

        Args:
            hint: Relevance hint

        Raises:
            InvalidParamError: relevance hint does not exist
        """
        if hint not in RELEVANCE_HINT:
            raise InvalidParamError(hint, RELEVANCE_HINT)
        self.setRelevanceHint(RELEVANCE_HINT[hint])

    def get_relevance_hint(self) -> str:
        """Return current relevance hint.

        Possible values: "unspecified", "distance", "lexical_place_name"

        Returns:
            Relevance hint
        """
        return RELEVANCE_HINT.inv[self.relevanceHint()]


if __name__ == "__main__":
    request = PlaceSearchRequest()
    print(bool(request))



# -*- coding: utf-8 -*-

from contextlib import contextmanager

from typing import Dict, Iterator, Callable, Optional, Union, Any

from qtpy import QtCore, QtGui, QtWidgets
import qstylizer.parser
import qstylizer.style
from deprecated import deprecated

from prettyqt import core, gui, widgets
from prettyqt.utils import bidict, colors, InvalidParamError, helpers, prettyprinter


CONTEXT_POLICIES = bidict(
    none=QtCore.Qt.NoContextMenu,
    prevent=QtCore.Qt.PreventContextMenu,
    default=QtCore.Qt.DefaultContextMenu,
    actions=QtCore.Qt.ActionsContextMenu,
    custom=QtCore.Qt.CustomContextMenu,
    # showhide_menu="showhide_menu",
)

MODALITIES = bidict(
    window=QtCore.Qt.WindowModal,
    application=QtCore.Qt.ApplicationModal,
    none=QtCore.Qt.NonModal,
)

CURSOR_SHAPES = bidict(
    arrow=QtCore.Qt.ArrowCursor,
    uparrow=QtCore.Qt.UpArrowCursor,
    cross=QtCore.Qt.CrossCursor,
    wait=QtCore.Qt.WaitCursor,
    caret=QtCore.Qt.IBeamCursor,
    size_vertical=QtCore.Qt.SizeVerCursor,
    size_horizonal=QtCore.Qt.SizeHorCursor,
    size_topright=QtCore.Qt.SizeBDiagCursor,
    size_topleft=QtCore.Qt.SizeFDiagCursor,
    size_all=QtCore.Qt.SizeAllCursor,
    blank=QtCore.Qt.BlankCursor,
    split_vertical=QtCore.Qt.SplitVCursor,
    split_horizontal=QtCore.Qt.SplitHCursor,
    pointing_hand=QtCore.Qt.PointingHandCursor,
    forbidden=QtCore.Qt.ForbiddenCursor,
    open_hand=QtCore.Qt.OpenHandCursor,
    closed_hand=QtCore.Qt.ClosedHandCursor,
    whats_this=QtCore.Qt.WhatsThisCursor,
    busy=QtCore.Qt.BusyCursor,
    drag_move=QtCore.Qt.DragMoveCursor,
    drag_copy=QtCore.Qt.DragCopyCursor,
    drag_link=QtCore.Qt.DragLinkCursor,
    bitmap=QtCore.Qt.BitmapCursor,
)

FOCUS_POLICIES = bidict(
    tab=QtCore.Qt.TabFocus,
    click=QtCore.Qt.ClickFocus,
    strong=QtCore.Qt.StrongFocus,
    wheel=QtCore.Qt.WheelFocus,
    none=QtCore.Qt.NoFocus,
)

WINDOW_FLAGS = bidict(
    frameless=QtCore.Qt.FramelessWindowHint,
    popup=QtCore.Qt.Popup,
    stay_on_top=QtCore.Qt.WindowStaysOnTopHint,
    tool=QtCore.Qt.Tool,
    window_title=QtCore.Qt.WindowTitleHint,
    customize_window=QtCore.Qt.CustomizeWindowHint,
)

ATTRIBUTES = bidict(
    native_window=QtCore.Qt.WA_NativeWindow,
    no_native_ancestors=QtCore.Qt.WA_DontCreateNativeAncestors,
)

QtWidgets.QWidget.__bases__ = (core.Object, QtGui.QPaintDevice)


class Widget(prettyprinter.PrettyPrinter, QtWidgets.QWidget):
    def __repr__(self) -> str:
        cls_name = self.__class__.__name__
        params = helpers.format_kwargs(self.serialize_fields())
        return f"{cls_name}({params})"

    def __setstate__(self, state: Dict[str, Any]) -> None:
        super().__init__()
        if self.layout() is None:
            self.set_layout(state["layout"])
        self.setSizePolicy(state["size_policy"])
        self.setAccessibleName(state["accessible_name"])
        self.setToolTip(state.get("tool_tip", ""))
        self.setStatusTip(state.get("status_tip", ""))

    def serialize_fields(self) -> Dict[str, Any]:
        icon = gui.Icon(self.windowIcon())
        return dict(
            layout=self.layout() if isinstance(self.layout(), widgets.Layout) else None,
            size_policy=self.get_size_policy(),
            actions=self.actions(),
            accessible_name=self.accessibleName(),
            tool_tip=self.toolTip(),
            tooltip_duration=self.toolTipDuration(),
            window_title=self.windowTitle(),
            enabled=self.isEnabled(),
            visible=self.isVisible(),
            stylesheet=self.styleSheet(),
            icon=icon if not icon.isNull() else None,
            modality=self.get_modality(),
            whats_this=self.whatsThis(),
            contextmenu_policy=self.get_contextmenu_policy(),
            focus_policy=self.get_focus_policy(),
            status_tip=self.statusTip(),
        )

    def resize(self, *size) -> None:
        if isinstance(size[0], tuple):
            super().resize(*size[0])
        else:
            super().resize(*size)

    def set_icon(self, icon: gui.icon.IconType) -> None:
        """Set the window icon.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon, color=colors.WINDOW_ICON_COLOR)
        self.setWindowIcon(icon)

    def set_min_size(self, *size) -> None:
        self.setMinimumSize(*size)

    def set_max_size(self, *size) -> None:
        self.setMaximumSize(*size)

    def set_min_width(self, width: Optional[int]) -> None:
        if width is None:
            width = 0
        self.setMinimumWidth(width)

    def set_max_width(self, width: Optional[int]) -> None:
        if width is None:
            width = QtWidgets.QWIDGETSIZE_MAX
        self.setMaximumWidth(width)

    def set_min_height(self, height: Optional[int]) -> None:
        if height is None:
            height = 0
        self.setMinimumHeight(height)

    def set_max_height(self, height: Optional[int]) -> None:
        if height is None:
            height = QtWidgets.QWIDGETSIZE_MAX
        self.setMaximumHeight(height)

    @property
    def title(self) -> str:
        return self.windowTitle()

    @title.setter
    def title(self, name: str):
        self.setWindowTitle(name)

    @property
    def enabled(self) -> bool:
        return self.isEnabled()

    @enabled.setter
    def enabled(self, state: bool):
        self.setEnabled(state)

    def set_enabled(self, enabled: bool = True) -> None:
        self.setEnabled(enabled)

    def set_disabled(self) -> None:
        self.setEnabled(False)

    def set_title(self, title: str) -> None:
        self.setWindowTitle(title)

    def set_tooltip(self, text: str) -> None:
        self.setToolTip(text)

    def set_font(
        self,
        font_name: Optional[str] = None,
        font_size: int = -1,
        weight: int = -1,
        italic: bool = False,
    ) -> gui.Font:
        if font_name is None:
            font_name = self.font().family()
        font = gui.Font(font_name, font_size, weight, italic)
        self.setFont(font)
        return font

    def set_window_flags(self, *flags: str, append: bool = False):
        for flag in flags:
            if flag not in WINDOW_FLAGS:
                raise InvalidParamError(flag, WINDOW_FLAGS)
        result = helpers.merge_flags(flags, WINDOW_FLAGS)
        if append:
            result = result | self.windowFlags()
        self.setWindowFlags(result)

    def set_flags(
        self,
        minimize: Optional[bool] = None,
        maximize: Optional[bool] = None,
        close: Optional[bool] = None,
        stay_on_top: Optional[bool] = None,
        frameless: Optional[bool] = None,
        window: Optional[bool] = None,
    ) -> None:
        if minimize is not None:
            self.setWindowFlag(QtCore.Qt.WindowMinimizeButtonHint, minimize)
        if maximize is not None:
            self.setWindowFlag(QtCore.Qt.WindowMaximizeButtonHint, maximize)
        if close is not None:
            self.setWindowFlag(QtCore.Qt.WindowCloseButtonHint, close)
        if stay_on_top is not None:
            self.setWindowFlag(QtCore.Qt.WindowStaysOnTopHint, stay_on_top)
        if frameless is not None:
            self.setWindowFlag(QtCore.Qt.FramelessWindowHint, frameless)
        if window is not None:
            self.setWindowFlag(QtCore.Qt.Window, window)

    def set_attribute(self, attribute: str, state: bool = True) -> None:
        if attribute not in ATTRIBUTES:
            raise InvalidParamError(attribute, ATTRIBUTES)
        self.setAttribute(ATTRIBUTES[attribute], state)

    def set_attributes(self, **kwargs: Dict[str, bool]) -> None:
        for attribute, state in kwargs.items():
            if attribute not in ATTRIBUTES:
                raise InvalidParamError(attribute, ATTRIBUTES)
            self.setAttribute(ATTRIBUTES[attribute], state)

    def set_modality(self, modality: str = "window") -> None:
        """Set modality for the dialog.

        Valid values for modality: "none", "window", "application"

        Args:
            modality: modality for the main window

        Raises:
            InvalidParamError: modality type does not exist
        """
        if modality not in MODALITIES:
            raise InvalidParamError(modality, MODALITIES)
        self.setWindowModality(MODALITIES[modality])

    def get_modality(self) -> str:
        """Get the current modality modes as a string.

        Possible values: "none", "window", "application"

        Returns:
            modality mode
            str
        """
        return MODALITIES.inv[self.windowModality()]

    def set_size_policy(
        self, horizontal: Optional[str] = None, vertical: Optional[str] = None
    ) -> None:
        """Set the sizes policy.

        possible values for both parameters are "fixed", "minimum", "maximum",
        "preferred", "expanding", "minimum_expanding" and "ignored"

        Args:
            horizontal: horizontal size policy
            vertical: vertical size policy
        """
        sp = self.get_size_policy()
        if horizontal is not None:
            sp.set_horizontal_policy(horizontal)
        if vertical is not None:
            sp.set_vertical_policy(vertical)
        self.setSizePolicy(sp)

    def get_size_policy(self) -> widgets.SizePolicy:
        qpol = self.sizePolicy()
        if isinstance(qpol, widgets.SizePolicy):
            return qpol
        pol = widgets.SizePolicy(
            qpol.horizontalPolicy(), qpol.verticalPolicy(), qpol.controlType()
        )
        pol.setHeightForWidth(qpol.hasHeightForWidth())
        pol.setWidthForHeight(qpol.hasWidthForHeight())
        pol.setHorizontalStretch(qpol.horizontalStretch())
        pol.setVerticalStretch(qpol.verticalStretch())
        return pol

    def get_palette(self) -> gui.Palette:
        return gui.Palette(self.palette())

    def set_background_color(self, color: colors.ColorType) -> None:
        col_str = "" if color is None else colors.get_color(color).name()
        with self.edit_stylesheet() as ss:
            ss.backgroundColor.setValue(col_str)

    @contextmanager
    def updates_off(self) -> Iterator[None]:
        self.setUpdatesEnabled(False)
        yield None
        self.setUpdatesEnabled(True)

    @contextmanager
    def edit_stylesheet(self) -> Iterator[qstylizer.style.StyleSheet]:
        ss = qstylizer.parser.parse(self.styleSheet())
        yield ss
        self.setStyleSheet(ss.toString())

    @contextmanager
    def current_font(self) -> Iterator[gui.Font]:
        font = gui.Font(self.font())
        yield font
        self.setFont(font)

    def set_contextmenu_policy(self, policy: str) -> None:
        """Set contextmenu policy for given item view.

        Allowed values are "none", "prevent", "default", "actions", "custom"

        Args:
            policy: contextmenu policy to use

        Raises:
            InvalidParamError: policy does not exist
        """
        if policy not in CONTEXT_POLICIES:
            raise InvalidParamError(policy, CONTEXT_POLICIES)
        self.setContextMenuPolicy(CONTEXT_POLICIES[policy])

    def get_contextmenu_policy(self) -> str:
        """Return current contextmenu policy.

        Possible values: "none", "prevent", "default", "actions", "custom"

        Returns:
            contextmenu policy
        """
        return CONTEXT_POLICIES.inv[self.contextMenuPolicy()]

    def set_custom_menu(self, method: Callable) -> None:
        self.set_contextmenu_policy("custom")
        self.customContextMenuRequested.connect(method)

    def set_layout(
        self,
        layout: Union[str, QtWidgets.QLayout, None],
        margin: Optional[int] = None,
        spacing: Optional[int] = None,
    ) -> None:
        if layout is None:
            return None
        if layout in ["horizontal", "vertical"]:
            self.box = widgets.BoxLayout(layout)
        elif layout == "grid":
            self.box = widgets.GridLayout()
        elif layout == "form":
            self.box = widgets.FormLayout()
        elif layout == "stacked":
            self.box = widgets.StackedLayout()
        elif layout == "flow":
            from prettyqt import custom_widgets

            self.box = custom_widgets.FlowLayout()
        elif isinstance(layout, QtWidgets.QLayout):
            self.box = layout
        else:
            raise ValueError("Invalid Layout")
        self.setLayout(self.box)
        if margin is not None:
            self.box.setContentsMargins(margin, margin, margin, margin)
        if spacing is not None:
            self.box.setSpacing(spacing)

    def center(self) -> None:
        qr = self.frameGeometry()
        cp = gui.GuiApplication.screens()[0].geometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

    def set_cursor(self, cursor: str) -> None:
        if cursor not in CURSOR_SHAPES:
            raise InvalidParamError(cursor, CURSOR_SHAPES)
        self.setCursor(CURSOR_SHAPES[cursor])

    def set_focus_policy(self, policy: str) -> None:
        """Set the way the widget accepts keyboard focus.

        Accepted values: "tab", "click", "strong", "wheel", "none"

        Args:
            policy (str): Focus policy

        Raises:
            InvalidParamError: Description
        """
        if policy not in FOCUS_POLICIES:
            raise InvalidParamError(policy, FOCUS_POLICIES)
        self.setFocusPolicy(FOCUS_POLICIES[policy])

    def get_focus_policy(self) -> str:
        """Return waay the widget accepts keyboard focus.

        Possible values:  "tab", "click", "strong", "wheel", "none"

        Returns:
            str: Focus policy
        """
        return FOCUS_POLICIES.inv[self.focusPolicy()]

    def set_font_size(self, size: int) -> None:
        font = self.font()
        font.setPointSize(size)
        self.setFont(font)

    @deprecated(reason="This method is deprecated, use get_font_metrics instead.")
    def font_metrics(self) -> gui.FontMetrics:
        return self.get_font_metrics()

    def get_font_metrics(self) -> gui.FontMetrics:
        return gui.FontMetrics(self.fontMetrics())

    def get_font_info(self) -> gui.FontInfo:
        return gui.FontInfo(self.fontInfo())

    def set_margin(self, margin: int) -> None:
        self.setContentsMargins(margin, margin, margin, margin)

    def raise_dock(self) -> bool:
        w = self.find_parent(QtWidgets.QDockWidget)
        if w is None:
            return False
        w.setVisible(True)
        w.raise_()
        return True


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = Widget()
    widget.show()
    app.main_loop()



from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QGraphicsTextItem.__bases__ = (widgets.GraphicsObject,)


class GraphicsTextItem(QtWidgets.QGraphicsTextItem):
    def __repr__(self):
        return f"{self.__class__.__name__}({repr(self.toPlainText())})"

    def serialize_fields(self):
        return dict(
            open_external_links=self.openExternalLinks(),
            text_width=self.textWidth(),
            plain_text=self.toPlainText(),
        )



# -*- coding: utf-8 -*-

from qtpy import QtLocation


from prettyqt import location
from prettyqt.utils import InvalidParamError


class PlaceContentRequest(QtLocation.QPlaceContentRequest):
    def set_content_type(self, typ: str):
        """Set the content type.

        Allowed values are "none", "image", "review", "editorial", "custom"

        Args:
            typ: Relevance type

        Raises:
            InvalidParamError: content type does not exist
        """
        if typ not in location.placecontent.TYPE:
            raise InvalidParamError(typ, location.placecontent.TYPE)
        self.setContentType(location.placecontent.TYPE[typ])

    def get_content_type(self) -> str:
        """Return current content type.

        Possible values: "none", "image", "review", "editorial", "custom"

        Returns:
            Relevance type
        """
        return location.placecontent.TYPE.inv[self.contentType()]


if __name__ == "__main__":
    request = PlaceContentRequest()
    print(bool(request))



# -*- coding: utf-8 -*-

from typing import Tuple, List

from qtpy import QtGui

from prettyqt import gui
from prettyqt.utils import bidict, InvalidParamError, prettyprinter


COORDINATE_MODES = bidict(
    logical=QtGui.QGradient.LogicalMode,
    object=QtGui.QGradient.ObjectMode,
    stretch_to_device=QtGui.QGradient.StretchToDeviceMode,
    object_bounding=QtGui.QGradient.ObjectBoundingMode,
)

SPREADS = bidict(
    pad=QtGui.QGradient.PadSpread,
    repeat=QtGui.QGradient.RepeatSpread,
    reflect=QtGui.QGradient.ReflectSpread,
)


TYPES = bidict(
    linear=QtGui.QGradient.LinearGradient,
    radial=QtGui.QGradient.RadialGradient,
    conical=QtGui.QGradient.ConicalGradient,
    none=QtGui.QGradient.NoGradient,
)


class Gradient(prettyprinter.PrettyPrinter, QtGui.QGradient):
    def __setitem__(self, key: float, value):
        self.setColorAt(key, value)

    def serialize_fields(self):
        return dict(
            coordinate_mode=self.get_coordinate_mode(),
            spread=self.get_spread(),
            stops=self.get_stops(),
        )

    def serialize(self):
        return self.serialize_fields()

    def set_coordinate_mode(self, mode: str):
        """Set the coordinate mode.

        Allowed values are "logical", "object", "stretch_to_device", "object_bounding"

        Args:
            mode: coordinate mode

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in COORDINATE_MODES:
            raise InvalidParamError(mode, COORDINATE_MODES)
        self.setCoordinateMode(COORDINATE_MODES[mode])

    def get_coordinate_mode(self) -> str:
        """Return current coordinate mode.

        Possible values: "logical", "object", "stretch_to_device", "object_bounding"

        Returns:
            coordinate mode
        """
        return COORDINATE_MODES.inv[self.coordinateMode()]

    def set_spread(self, method: str):
        """Set the spread method.

        Allowed values are "pad", "repeat", "reflect"

        Args:
            method: spread method

        Raises:
            InvalidParamError: method does not exist
        """
        if method not in SPREADS:
            raise InvalidParamError(method, SPREADS)
        self.setSpread(SPREADS[method])

    def get_spread(self) -> str:
        """Return current spread method.

        Possible values: "pad", "repeat", "reflect"

        Returns:
            spread method
        """
        return SPREADS.inv[self.spread()]

    def get_type(self) -> str:
        """Return current gradient type.

        Possible values: "linear", "radial", "conical", "none"

        Returns:
            gradient type
        """
        return TYPES.inv[self.type()]

    def get_stops(self) -> List[Tuple[float, gui.Color]]:
        return [(i, gui.Color(j)) for (i, j) in self.stops()]


if __name__ == "__main__":
    grad = Gradient()
    grad.setStops([(0.0, gui.Color("red")), (1.0, gui.Color("green"))])
    print(grad.get_stops())
    print(repr(grad))



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets

QtWidgets.QGraphicsScale.__bases__ = (widgets.GraphicsTransform,)


class GraphicsScale(QtWidgets.QGraphicsScale):
    pass


if __name__ == "__main__":
    transform = GraphicsScale()



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt.utils import bidict, InvalidParamError

FIELD_ALIGNMENT = bidict(
    left=QtCore.QTextStream.AlignLeft,
    right=QtCore.QTextStream.AlignRight,
    center=QtCore.QTextStream.AlignCenter,
    accounting_style=QtCore.QTextStream.AlignAccountingStyle,
)

NUMBER_FLAGS = bidict(
    show_base=QtCore.QTextStream.ShowBase,
    force_point=QtCore.QTextStream.ForcePoint,
    force_sign=QtCore.QTextStream.ForceSign,
    uppercase_base=QtCore.QTextStream.UppercaseBase,
    uppercase_digits=QtCore.QTextStream.UppercaseDigits,
)

REAL_NUMBER_NOTATION = bidict(
    scientific=QtCore.QTextStream.ScientificNotation,
    fixed=QtCore.QTextStream.FixedNotation,
    smart=QtCore.QTextStream.SmartNotation,
)

STATUS = bidict(
    ok=QtCore.QTextStream.Ok,
    read_past_end=QtCore.QTextStream.ReadPastEnd,
    read_corrupt_data=QtCore.QTextStream.ReadCorruptData,
    write_failed=QtCore.QTextStream.WriteFailed,
)


class TextStream(QtCore.QTextStream):
    def set_field_alignment(self, alignment: str):
        """Set the field alignment.

        Valid values are "left", "right", "center", "accounting_style"

        Args:
            alignment: field alignment

        Raises:
            InvalidParamError: invalid field alignment
        """
        if alignment not in FIELD_ALIGNMENT:
            raise InvalidParamError(alignment, FIELD_ALIGNMENT)
        self.setFieldAlignment(FIELD_ALIGNMENT[alignment])

    def get_field_alignment(self) -> str:
        """Get current field alignment.

        Possible values are "left", "right", "center", "accounting_style"

        Returns:
            current field alignment
        """
        return FIELD_ALIGNMENT.inv[self.fieldAlignment()]

    def set_status(self, status: str):
        """Set the status.

        Valid values are "ok", "read_past_end", "read_corrupt_data", "write_failed"

        Args:
            status: status

        Raises:
            InvalidParamError: invalid status
        """
        if status not in STATUS:
            raise InvalidParamError(status, STATUS)
        self.setStatus(STATUS[status])

    def get_status(self) -> str:
        """Get current status.

        Possible values are "ok", "read_past_end", "read_corrupt_data", "write_failed"

        Returns:
            current status
        """
        return STATUS.inv[self.status()]

    def set_real_number_notation(self, notation: str):
        """Set the real number notation.

        Valid values are "scientific", "fixed", "smart"

        Args:
            notation: real number notation

        Raises:
            InvalidParamError: invalid real number notation
        """
        if notation not in REAL_NUMBER_NOTATION:
            raise InvalidParamError(notation, REAL_NUMBER_NOTATION)
        self.setRealNumberNotation(REAL_NUMBER_NOTATION[notation])

    def get_real_number_notation(self) -> str:
        """Get current real number notation.

        Possible values are "scientific", "fixed", "smart"

        Returns:
            current real number notation
        """
        return REAL_NUMBER_NOTATION.inv[self.realNumberNotation()]


if __name__ == "__main__":
    matcher = TextStream("Test")
    print(repr(matcher))



# -*- coding: utf-8 -*-

from typing import List, Iterator

from qtpy import QtNetwork

from prettyqt import core


QtNetwork.QNetworkCookieJar.__bases__ = (core.Object,)


class NetworkCookieJar(QtNetwork.QNetworkCookieJar):
    def __getitem__(self, url: str) -> List[QtNetwork.QNetworkCookie]:
        url = core.Url(url)
        return self.cookiesForUrl(url)

    def __repr__(self):
        return "NetworkCookieJar()"

    def __iter__(self) -> Iterator[QtNetwork.QNetworkCookie]:
        return iter(self.allCookies())



from typing import Any

from qtpy import QtGui

from prettyqt.utils import bidict, InvalidParamError


IMAGE_OPTION = bidict(
    size=QtGui.QImageIOHandler.Size,
    clip_rect=QtGui.QImageIOHandler.ClipRect,
    scaled_size=QtGui.QImageIOHandler.ScaledSize,
    scaled_clip_rect=QtGui.QImageIOHandler.ScaledClipRect,
    description=QtGui.QImageIOHandler.Description,
    compression_ratio=QtGui.QImageIOHandler.CompressionRatio,
    gamma=QtGui.QImageIOHandler.Gamma,
    quality=QtGui.QImageIOHandler.Quality,
    name=QtGui.QImageIOHandler.Name,
    subtype=QtGui.QImageIOHandler.SubType,
    incremental_reading=QtGui.QImageIOHandler.IncrementalReading,
    endianness=QtGui.QImageIOHandler.Endianness,
    animation=QtGui.QImageIOHandler.Animation,
    background_color=QtGui.QImageIOHandler.BackgroundColor,
    # image_format=QtGui.QImageIOHandler.ImageFormat,
    supported_sub_types=QtGui.QImageIOHandler.SupportedSubTypes,
    optimized_write=QtGui.QImageIOHandler.OptimizedWrite,
    progressive_scan_write=QtGui.QImageIOHandler.ProgressiveScanWrite,
    image_transformation=QtGui.QImageIOHandler.ImageTransformation,
    transformation_by_default=QtGui.QImageIOHandler.TransformedByDefault,
)

TRANSFORMATIONS = bidict(
    none=QtGui.QImageIOHandler.TransformationNone,
    mirror=QtGui.QImageIOHandler.TransformationMirror,
    flip=QtGui.QImageIOHandler.TransformationFlip,
    rotate_180=QtGui.QImageIOHandler.TransformationRotate180,
    roate_90=QtGui.QImageIOHandler.TransformationRotate90,
    mirror_and_rotate_90=QtGui.QImageIOHandler.TransformationMirrorAndRotate90,
    flip_and_rotate_90=QtGui.QImageIOHandler.TransformationFlipAndRotate90,
    rotate_270=QtGui.QImageIOHandler.TransformationRotate270,
)


class ImageIOHandler(QtGui.QImageIOHandler):
    def __getitem__(self, key: str):
        return self.get_option(key)

    def __setitem__(self, key: str, value):
        self.set_option(key, value)

    def get_format(self) -> str:
        return bytes(self.format()).decode()

    def set_option(self, option: str, value):
        """Set option to given value.

        Allowed values are "size", "clip_rect", "scaled_size", "scaled_clip_rect",
                           "description", "compression_ratio", "gamma", "quality",
                           "name", "subtype", "incremental_reading", "endianness",
                           "animation", "background_color", "image_format",
                           "supported_sub_types", "optimized_write",
                           "progressive_scan_write", "image_transformation",
                           "transformation_by_default"

        Args:
            option: option to use
            value: value to set

        Raises:
            InvalidParamError: option does not exist
        """
        if option not in IMAGE_OPTION:
            raise InvalidParamError(option, IMAGE_OPTION)
        self.setOption(IMAGE_OPTION[option], value)

    def get_option(self, option: str) -> Any:
        """Return the value assigned to option.

        Possible values: "size", "clip_rect", "scaled_size", "scaled_clip_rect",
                         "description", "compression_ratio", "gamma", "quality",
                         "name", "subtype", "incremental_reading", "endianness",
                         "animation", "background_color", "image_format",
                         "supported_sub_types", "optimized_write",
                         "progressive_scan_write", "image_transformation",
                         "transformation_by_default"

        Args:
            option: option to get

        Returns:
            option
        """
        if option not in IMAGE_OPTION:
            raise InvalidParamError(option, IMAGE_OPTION)
        return self.option(IMAGE_OPTION[option])

    def supports_option(self, option: str) -> bool:
        """Return whether the image handler supports given option.

        Possible values: "size", "clip_rect", "scaled_size", "scaled_clip_rect",
                         "description", "compression_ratio", "gamma", "quality",
                         "name", "subtype", "incremental_reading", "endianness",
                         "animation", "background_color", "image_format",
                         "supported_sub_types", "optimized_write",
                         "progressive_scan_write", "image_transformation",
                         "transformation_by_default"

        Args:
            option: option to check

        Returns:
            option
        """
        if option not in IMAGE_OPTION:
            raise InvalidParamError(option, IMAGE_OPTION)
        return self.supportsOption(IMAGE_OPTION[option])


if __name__ == "__main__":
    writer = ImageIOHandler()



# -*- coding: utf-8 -*-

from qtpy import QtGui, QtCore

from prettyqt import gui
from prettyqt.utils import bidict, InvalidParamError


CAP_STYLES = bidict(
    flat=QtCore.Qt.FlatCap, square=QtCore.Qt.SquareCap, round=QtCore.Qt.RoundCap
)

JOIN_STYLES = bidict(
    miter=QtCore.Qt.MiterJoin,
    bevel=QtCore.Qt.BevelJoin,
    round=QtCore.Qt.RoundJoin,
    svg_miter=QtCore.Qt.SvgMiterJoin,
)


class PainterPathStroker(QtGui.QPainterPathStroker):
    def set_cap_style(self, style: str):
        """Set cap style to use.

        Allowed values are "flat", "square", "round"

        Args:
            style: cap style to use

        Raises:
            InvalidParamError: cap style does not exist
        """
        if style not in CAP_STYLES:
            raise InvalidParamError(style, CAP_STYLES)
        self.setCapStyle(CAP_STYLES[style])

    def get_cap_style(self) -> str:
        """Return current cap style.

        Possible values: "flat", "square", "round"

        Returns:
            cap style
        """
        return CAP_STYLES.inv[self.capStyle()]

    def set_join_style(self, style: str):
        """Set join style to use.

        Allowed values are "miter", "bevel", "round", "svg_miter"

        Args:
            style: join style to use

        Raises:
            InvalidParamError: join style does not exist
        """
        if style not in JOIN_STYLES:
            raise InvalidParamError(style, JOIN_STYLES)
        self.setJoinStyle(JOIN_STYLES[style])

    def get_join_style(self) -> str:
        """Return current join style.

        Possible values: "miter", "bevel", "round", "svg_miter"

        Returns:
            join style
        """
        return JOIN_STYLES.inv[self.joinStyle()]

    def create_stroke(self, path: QtGui.QPainterPath) -> gui.PainterPath:
        return gui.PainterPath(self.createStroke(path))


if __name__ == "__main__":
    p = PainterPathStroker(QtCore.QPoint(1, 1))
    print(list(p))



# -*- coding: utf-8 -*-

from __future__ import annotations

import pathlib
from typing import Union, Optional

from qtpy import QtCore, QtWidgets

from prettyqt import gui, widgets


class Image(widgets.Label):
    def __init__(
        self,
        path: Union[pathlib.Path, str] = None,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(parent=parent)
        if path:
            self.set_image(path)

    def __repr__(self):
        return f"Image({self.text()!r})"

    def __bool__(self):
        return not self.isNull()

    def serialize_fields(self):
        return dict(
            text=self.text(),
            scaled_contents=self.hasScaledContents(),
            margin=self.margin(),
            alignment=int(self.alignment()),
        )

    def __setstate__(self, state):
        self.__init__()
        self.setText(state.get("text", ""))
        self.setMargin(state.get("margin", 0))
        self.setAlignment(QtCore.Qt.Alignment(state.get("alignment")))
        self.setScaledContents(state["scaled_contents"])

    def set_image(self, path: Union[pathlib.Path, str], width: int = 300):
        self.setScaledContents(True)
        self.set_alignment(horizontal="center")
        self.setText(
            "<html><head/><body><p>"
            f"<img src={str(path)!r} width={str(width)!r}/>"
            "</p></body></html>"
        )

    @classmethod
    def from_path(cls, path: Union[pathlib.Path, str], parent=None) -> Image:
        pixmap = gui.Pixmap.from_file(path)
        label = cls(parent=parent)
        label.setPixmap(pixmap)
        label.resize(pixmap.width(), pixmap.height())
        return label


if __name__ == "__main__":
    app = widgets.app()
    widget = Image("https://act-crm-addon.de/wp-content/uploads/2018/12/test.png")
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-


from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineWidgets

from prettyqt.utils import bidict, InvalidParamError


INJECTION_POINTS = bidict(
    document_creation=QtWebEngineWidgets.QWebEngineScript.DocumentCreation,
    document_ready=QtWebEngineWidgets.QWebEngineScript.DocumentReady,
    deferred=QtWebEngineWidgets.QWebEngineScript.Deferred,
)

SCRIPT_WORLD_IDS = bidict(
    main_world=QtWebEngineWidgets.QWebEngineScript.MainWorld,
    application_world=QtWebEngineWidgets.QWebEngineScript.ApplicationWorld,
    user_world=QtWebEngineWidgets.QWebEngineScript.UserWorld,
)


class WebEngineScript(QtWebEngineWidgets.QWebEngineScript):
    def set_injection_point(self, point: str):
        """Set injection point.

        Allowed values are "document_creation", "document_ready", "deferred"

        Args:
            point: injection point to use

        Raises:
            InvalidParamError: injection point does not exist
        """
        if point not in INJECTION_POINTS:
            raise InvalidParamError(point, INJECTION_POINTS)
        self.setInjectionPoint(INJECTION_POINTS[point])

    def get_injection_point(self) -> str:
        """Return injection point.

        Possible values: "document_creation", "document_ready", "deferred"

        Returns:
            injection point
        """
        return INJECTION_POINTS.inv[self.injectionPoint()]


if __name__ == "__main__":
    item = WebEngineScript()



# -*- coding: utf-8 -*-

"""Location module."""

from .georouterequest import GeoRouteRequest
from .placeattribute import PlaceAttribute
from .placeratings import PlaceRatings
from .placecontactdetail import PlaceContactDetail
from .placeicon import PlaceIcon
from .placesupplier import PlaceSupplier
from .placecategory import PlaceCategory
from .placeuser import PlaceUser
from .placecontent import PlaceContent
from .placeimage import PlaceImage
from .placereview import PlaceReview
from .placeeditorial import PlaceEditorial
from .geocodingmanager import GeoCodingManager
from .georoutingmanager import GeoRoutingManager
from .geomaneuver import GeoManeuver
from .georoutesegment import GeoRouteSegment
from .georoute import GeoRoute
from .georouteleg import GeoRouteLeg
from .placesearchrequest import PlaceSearchRequest
from .placecontentrequest import PlaceContentRequest
from .place import Place
from .placematchrequest import PlaceMatchRequest
from .placesearchresult import PlaceSearchResult
from .placeresult import PlaceResult
from .placeproposedsearchresult import PlaceProposedSearchResult
from .placereply import PlaceReply
from .placesearchreply import PlaceSearchReply
from .placecontentreply import PlaceContentReply
from .placedetailsreply import PlaceDetailsReply
from .placematchreply import PlaceMatchReply
from .placeidreply import PlaceIdReply
from .placemanager import PlaceManager
from .geoserviceprovider import GeoServiceProvider


__all__ = [
    "PlaceAttribute",
    "PlaceContactDetail",
    "PlaceCategory",
    "PlaceRatings",
    "PlaceUser",
    "Place",
    "PlaceSupplier",
    "PlaceManager",
    "PlaceIcon",
    "PlaceContent",
    "PlaceImage",
    "PlaceReview",
    "PlaceEditorial",
    "GeoCodingManager",
    "GeoRoutingManager",
    "GeoManeuver",
    "GeoRoute",
    "GeoRouteLeg",
    "GeoRouteRequest",
    "GeoRouteSegment",
    "PlaceSearchRequest",
    "PlaceContentRequest",
    "PlaceMatchRequest",
    "GeoServiceProvider",
    "PlaceReply",
    "PlaceSearchReply",
    "PlaceContentReply",
    "PlaceDetailsReply",
    "PlaceMatchReply",
    "PlaceIdReply",
    "PlaceSearchResult",
    "PlaceResult",
    "PlaceProposedSearchResult",
]



# -*- coding: utf-8 -*-

from typing import Optional, Pattern, Union

from qtpy import QtWidgets
import regex as re

from prettyqt import core, custom_validators, custom_widgets, widgets

MAP = dict(
    multiline=re.MULTILINE,
    ignorecase=re.IGNORECASE,
    ascii=re.ASCII,
    dotall=re.DOTALL,
    verbose=re.VERBOSE,
)


class RegexInput(widgets.Widget):

    value_changed = core.Signal(object)

    def __init__(
        self,
        show_flags: bool = True,
        show_error: bool = True,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(parent=parent)
        self.set_layout("grid")
        self.lineedit = custom_widgets.SingleLineTextEdit()
        self.lineedit.set_syntaxhighlighter("regex")
        self.tb_flags = custom_widgets.BoolDictToolButton(
            "Flags", icon="mdi.flag-variant-outline"
        )
        self.label_error = widgets.Label()
        self.label_error.set_color("red")
        self.box[0, 0:1] = self.lineedit
        if show_flags:
            self.box[0, 2] = self.tb_flags
        if show_error:
            self.box[1, 0:2] = self.label_error
        val = custom_validators.RegexPatternValidator()
        val.error_occured.connect(self.label_error.set_text)
        val.pattern_updated.connect(self.value_changed)
        self.tb_flags.value_changed.connect(self._on_value_change)
        self.lineedit.set_validator(val)
        dct = dict(
            multiline="MultiLine",
            ignorecase="Ignore case",
            ascii="ASCII-only matching",
            dotall="Dot matches newline",
            verbose="Ignore whitespace",
        )
        self.tb_flags.set_dict(dct)

    def _on_value_change(self):
        val = self.get_value()
        self.value_changed.emit(val)

    @property
    def pattern(self) -> str:
        return self.lineedit.text()

    @pattern.setter
    def pattern(self, value: str):
        self.lineedit.set_text(value)

    @property
    def compile_flags(self) -> int:
        ret_val = 0
        for identifier, flag in MAP.items():
            if self.tb_flags[identifier]:
                ret_val |= flag
        return ret_val

    @compile_flags.setter
    def compile_flags(self, value: int):
        for identifier, flag in MAP.items():
            self.tb_flags[identifier] = bool(value & flag)

    def set_value(self, value: Optional[Union[str, Pattern]]):
        if value is None:
            self.pattern = ""
            self.flags = 0
        elif isinstance(value, str):
            self.pattern = value
            self.flags = 0
        else:
            self.pattern = value.pattern
            self.compile_flags = value.flags

    def get_value(self) -> Pattern:
        return re.compile(self.pattern, self.compile_flags)

    def is_valid(self) -> bool:
        return self.lineedit.is_valid()


if __name__ == "__main__":
    app = widgets.app()
    widget = RegexInput(show_flags=True, show_error=True)
    widget.show()
    widget.value_changed.connect(print)
    app.main_loop()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyleOptionFocusRect.__bases__ = (widgets.StyleOption,)


class StyleOptionFocusRect(QtWidgets.QStyleOptionFocusRect):
    pass



# -*- coding: utf-8 -*-

"""bluetooth module.

contains QtBluetooth-based classes
"""

from .bluetoothuuid import BluetoothUuid
from .bluetoothaddress import BluetoothAddress
from .bluetoothserviceinfo import BluetoothServiceInfo
from .bluetoothdevicediscoveryagent import BluetoothDeviceDiscoveryAgent
from .bluetoothservicediscoveryagent import BluetoothServiceDiscoveryAgent


__all__ = [
    "BluetoothUuid",
    "BluetoothAddress",
    "BluetoothServiceInfo",
    "BluetoothDeviceDiscoveryAgent",
    "BluetoothServiceDiscoveryAgent",
]



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict


TYPE = bidict(
    read=QtCore.QSocketNotifier.Read,
    write=QtCore.QSocketNotifier.Write,
    exception=QtCore.QSocketNotifier.Exception,
)


QtCore.QSocketNotifier.__bases__ = (core.Object,)


class SocketNotifier(QtCore.QSocketNotifier):
    # def __repr__(self):
    #     return f"{type(self).__name__}({self.socket()}, {self.type()})"

    def get_type(self) -> str:
        """Return socket event type.

        Possible values: "read", "write", "exception"

        Returns:
            socket event type
        """
        return TYPE.inv[self.type()]


if __name__ == "__main__":
    app = core.app()
    import os

    print("hre")
    read, write = os.pipe()
    notifier = SocketNotifier(read, QtCore.QSocketNotifier.Read)
    print(repr(notifier))
    app.main_loop()



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core


QtCore.QAbstractTableModel.__bases__ = (core.AbstractItemModel,)


class AbstractTableModel(QtCore.QAbstractTableModel):
    pass



from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QGraphicsSimpleTextItem.__bases__ = (widgets.AbstractGraphicsShapeItem,)


class GraphicsSimpleTextItem(QtWidgets.QGraphicsSimpleTextItem):
    def __repr__(self):
        return f"{self.__class__.__name__}({repr(self.text())})"



# -*- coding: utf-8 -*-

from qtpy import QtMultimediaWidgets, QtCore

from prettyqt import core, multimedia, widgets
from prettyqt.utils import bidict, InvalidParamError


ASPECT_RATIO_MODES = bidict(
    ignore=QtCore.Qt.IgnoreAspectRatio,
    keep=QtCore.Qt.KeepAspectRatio,
    keep_by_expanding=QtCore.Qt.KeepAspectRatioByExpanding,
)

QtMultimediaWidgets.QGraphicsVideoItem.__bases__ = (
    widgets.GraphicsObject,
    multimedia.MediaBindableInterface,
)


class GraphicsVideoItem(QtMultimediaWidgets.QGraphicsVideoItem):
    def get_offset(self) -> core.PointF:
        return core.PointF(self.offset())

    def get_native_size(self) -> core.SizeF:
        return core.SizeF(self.nativeSize())

    def get_size(self) -> core.SizeF:
        return core.SizeF(self.size())

    def set_aspect_ratio_mode(self, mode: str):
        """Set the aspect ratio mode.

        Allowed values are "ignore", "keep", "keep_by_expanding"

        Args:
            mode: aspect ratio mode

        Raises:
            InvalidParamError: aspect ratio mode does not exist
        """
        if mode not in ASPECT_RATIO_MODES:
            raise InvalidParamError(mode, ASPECT_RATIO_MODES)
        self.setAspectRatioMode(ASPECT_RATIO_MODES[mode])

    def get_aspect_ratio_mode(self) -> str:
        """Return current aspect ratio mode.

        Possible values: "ignore", "keep", "keep_by_expanding"

        Returns:
            aspect ratio mode
        """
        return ASPECT_RATIO_MODES.inv[self.aspectRatioMode()]



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyledItemDelegate.__bases__ = (widgets.AbstractItemDelegate,)


class StyledItemDelegate(QtWidgets.QStyledItemDelegate):
    pass



# -*- coding: utf-8 -*-

from __future__ import annotations

from qtpy import QtLocation

from prettyqt import location


QtLocation.QPlaceContentReply.__bases__ = (location.PlaceReply,)


class PlaceContentReply(QtLocation.QPlaceContentReply):
    def __len__(self):
        return self.totalCount()

    @classmethod
    def clone_from(cls, obj: QtLocation.QPlaceContentReply) -> PlaceContentReply:
        reply = cls(obj.parent())
        reply.setContent(obj.content())
        reply.setTotalCount(obj.totalCount())
        request = location.PlaceContentRequest(obj.nextPageRequest())
        reply.setNextPageRequest(request)
        request = location.PlaceContentRequest(obj.previousPageRequest())
        reply.setPreviousPageRequest(request)
        request = location.PlaceContentRequest(obj.request())
        reply.setRequest(request)
        return reply

    def get_next_page_request(self) -> location.PlaceContentRequest:
        return location.PlaceContentRequest(self.nextPageRequest())

    def get_previous_page_request(self) -> location.PlaceContentRequest:
        return location.PlaceContentRequest(self.previousPageRequest())

    def get_request(self) -> location.PlaceContentRequest:
        return location.PlaceContentRequest(self.request())


if __name__ == "__main__":
    reply = PlaceContentReply()



# -*- coding: utf-8 -*-

from qtpy import QtCore


class PointF(QtCore.QPointF):
    def __repr__(self):
        return f"PointF(x={self.x()}, y={self.y()})"

    def __getitem__(self, index) -> float:
        return (self.x(), self.y())[index]



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui

QtGui.QTextFrame.__bases__ = (gui.TextObject,)


class TextFrame(QtGui.QTextFrame):
    def __repr__(self):
        return "TextFrame()"

    def get_first_cursor_position(self) -> gui.TextCursor:
        return gui.TextCursor(self.firstCursorPosition())

    def get_last_cursor_position(self) -> gui.TextCursor:
        return gui.TextCursor(self.lastCursorPosition())


if __name__ == "__main__":
    doc = TextFrame()



# -*- coding: utf-8 -*-

from prettyqt import widgets
from prettyqt.utils import bidict


class MappedCheckBox(widgets.CheckBox):
    def __init__(self, *args, true_value=True, false_value=False, **kwargs):
        super().__init__(*args, **kwargs)
        dct = {True: true_value, False: false_value}
        self.map = bidict(dct)

    def serialize_fields(self):
        return dict(map=self.map)

    def get_value(self):
        return self.map[self.isChecked()]

    def set_value(self, value):
        val = self.map.inv[value]
        super().set_value(val)


if __name__ == "__main__":
    app = widgets.app()
    widget = MappedCheckBox("Test")
    widget.show()
    app.main_loop()
    print(widget.get_value())



# -*- coding: utf-8 -*-

from __future__ import annotations

from qtpy import QtCore
from qtpy.QtCharts import QtCharts

from prettyqt import charts, gui


QtCharts.QXYSeries.__bases__ = (charts.AbstractSeries,)


class XYSeries(QtCharts.QXYSeries):
    """QXYSeries with some custom properties."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # self._process_name = ""
        self.setUseOpenGL()

    def __setitem__(self, index: int, val: QtCore.QPointF):
        self.replace(index, val)

    def __delitem__(self, index: int):
        self.remove(index)

    def __setstate__(self, state):
        self.__init__()
        super().append(state["points"])

    def __add__(self, other: QtCore.QPointF) -> XYSeries:
        self.append(other)
        return self

    def serialize_fields(self):
        return dict(points=self.pointsVector())

    def get_pen(self) -> gui.Pen:
        return gui.Pen(self.pen())

    def get_brush(self) -> gui.Brush:
        return gui.Brush(self.brush())



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict

CORNER_WIDGETS = bidict(
    none=QtWidgets.QStyleOptionTab.NoCornerWidgets,
    left=QtWidgets.QStyleOptionTab.LeftCornerWidget,
    right=QtWidgets.QStyleOptionTab.RightCornerWidget,
)

SELECTED_POSITION = bidict(
    not_adjacent=QtWidgets.QStyleOptionTab.NotAdjacent,
    next_is_selected=QtWidgets.QStyleOptionTab.NextIsSelected,
    previous_is_selected=QtWidgets.QStyleOptionTab.PreviousIsSelected,
)

TAB_FEATURE = bidict(
    none=QtWidgets.QStyleOptionTab.TabFeature(0),
    has_frame=QtWidgets.QStyleOptionTab.HasFrame,
)

TAB_POSITION = bidict(
    beginning=QtWidgets.QStyleOptionTab.Beginning,
    middle=QtWidgets.QStyleOptionTab.Middle,
    end=QtWidgets.QStyleOptionTab.End,
    only_one_tab=QtWidgets.QStyleOptionTab.OnlyOneTab,
)


QtWidgets.QStyleOptionTab.__bases__ = (widgets.StyleOption,)


class StyleOptionTab(QtWidgets.QStyleOptionTab):
    pass



# -*- coding: utf-8 -*-

import pathlib

from qtpy import QtCore


class Dir(QtCore.QDir):
    def __getattr__(self, attr: str):
        return getattr(self.to_path(), attr)

    def __repr__(self):
        return f"Dir({self.absolutePath()!r})"

    def __str__(self):
        return self.absolutePath()

    def __reduce__(self):
        return (self.__class__, (self.absolutePath(),))

    def __truediv__(self, other: str) -> pathlib.Path:
        path = self.to_path() / other
        return path

    def to_path(self) -> pathlib.Path:
        return pathlib.Path(self.absolutePath())



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui


QtGui.QPaintDeviceWindow.__bases__ = (gui.Window, gui.PaintDevice)


class PaintDeviceWindow(QtGui.QPaintDeviceWindow):
    pass


if __name__ == "__main__":
    window = PaintDeviceWindow()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import core, widgets


QtWidgets.QGraphicsObject.__bases__ = (core.Object, widgets.GraphicsItem)


class GraphicsObject(QtWidgets.QGraphicsObject):
    def serialize_fields(self):
        return dict(
            enabled=self.isEnabled(),
            opacity=self.opacity(),
            pos=core.PointF(self.pos()),
            # z=self.z(),
            rotation=self.rotation(),
            scale=self.scale(),
            transform_origin_point=core.PointF(self.transformOriginPoint()),
            visible=self.isVisible(),
            graphics_effect=self.graphicsEffect(),
        )



# -*- coding: utf-8 -*-

from qtpy import QtGui


class TextOption(QtGui.QTextOption):
    def set_text(self, text: str):
        self.setPlainText(text)


if __name__ == "__main__":
    doc = TextOption()



# -*- coding: utf-8 -*-

import pathlib

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import InvalidParamError


class FileInfo(QtCore.QFileInfo):
    def __init__(self, *args, **kwargs):
        if len(args) == 1 and isinstance(args[0], pathlib.Path):
            super().__init__(str(args[0]))
        else:
            super().__init__(*args, **kwargs)

    def __repr__(self):
        return f"FileInfo({self.absoluteFilePath()!r})"

    def __str__(self):
        return self.absoluteFilePath()

    def __getattr__(self, attr: str):
        return getattr(self.get_absolute_file_path(), attr)

    def get_dir(self) -> pathlib.Path:
        return pathlib.Path(self.dir().absolutePath())

    def get_absolute_file_path(self) -> pathlib.Path:
        return pathlib.Path(self.absoluteFilePath())

    def get_birth_time(self) -> core.DateTime:
        return core.DateTime(self.birthTime())

    def get_metadata_change_time(self) -> core.DateTime:
        return core.DateTime(self.metadataChangeTime())

    def get_last_modified(self) -> core.DateTime:
        return core.DateTime(self.lastModified())

    def get_last_read(self) -> core.DateTime:
        return core.DateTime(self.lastRead())

    def get_file_time(self, typ: str) -> core.DateTime:
        if typ not in core.filedevice.FILE_TIME:
            raise InvalidParamError(typ, core.filedevice.FILE_TIME)
        return core.DateTime(self.fileTime(core.filedevice.FILE_TIME[typ]))


if __name__ == "__main__":
    p = pathlib.Path.home()
    f = FileInfo(p)



# -*- coding: utf-8 -*-

from typing import List

from qtpy import QtCore
from qtpy.QtCharts import QtCharts

from prettyqt import core, widgets, charts
from prettyqt.utils import bidict, InvalidParamError


THEMES = bidict(
    {
        "Light": QtCharts.QChart.ChartThemeLight,
        "Blue Cerulean": QtCharts.QChart.ChartThemeBlueCerulean,
        "Dark": QtCharts.QChart.ChartThemeDark,
        "Brown Sand": QtCharts.QChart.ChartThemeBrownSand,
        "Blue NCS": QtCharts.QChart.ChartThemeBlueNcs,
        "High Contrast": QtCharts.QChart.ChartThemeHighContrast,
        "Blue Icy": QtCharts.QChart.ChartThemeBlueIcy,
        "Qt": QtCharts.QChart.ChartThemeQt,
    }
)

ANIMATION_OPTIONS = bidict(
    none=QtCharts.QChart.NoAnimation,
    grid_axis=QtCharts.QChart.GridAxisAnimations,
    series=QtCharts.QChart.SeriesAnimations,
    all=QtCharts.QChart.AllAnimations,
)

CHART_TYPES = bidict(
    undefined=QtCharts.QChart.ChartTypeUndefined,
    cartesian=QtCharts.QChart.ChartTypeCartesian,
    polar=QtCharts.QChart.ChartTypePolar,
)

ALIGNMENTS = dict(
    left=QtCore.Qt.AlignLeft,
    right=QtCore.Qt.AlignRight,
    top=QtCore.Qt.AlignTop,
    bottom=QtCore.Qt.AlignBottom,
)


QtCharts.QChart.__bases__ = (widgets.GraphicsWidget,)


class Chart(QtCharts.QChart):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.max_x = 0
        self.max_y = 0
        self.min_x = 0
        self.min_y = 0

    def serialize_fields(self):
        return dict(
            animation_duration=self.animationDuration(),
            animation_easing_curve=self.get_animation_easing_curve(),
            animation_options=self.get_animation_options(),
            background_roundness=self.backgroundRoundness(),
            background_visible=self.isBackgroundVisible(),
            chart_type=self.get_chart_type(),
            drop_shadow_enabled=self.isDropShadowEnabled(),
            locale=self.get_locale(),
            localize_numbers=self.localizeNumbers(),
            margins=self.get_margins(),
            plot_area=self.get_plot_area(),
            plot_area_background_visible=self.isPlotAreaBackgroundVisible(),
            theme=self.get_theme(),
            title=self.title(),
        )

    def update_boundaries(self):
        """Set new min/max values based on axis."""
        self.max_x = self.axisX().max()
        self.max_y = self.axisY().max()
        self.min_x = self.axisX().min()
        self.min_y = self.axisY().min()

    def hide_legend(self):
        self.legend().hide()

    def show_legend(self):
        self.legend().show()

    def get_legend(self):
        return charts.Legend(self.legend())

    def set_legend_alignment(self, alignment: str):
        if alignment not in ALIGNMENTS:
            raise InvalidParamError(alignment, ALIGNMENTS)
        self.legend().setAlignment(ALIGNMENTS[alignment])

    def set_theme(self, theme_name: str):
        self.setTheme(THEMES[theme_name])

    def set_animation_options(self, option: str):
        self.setAnimationOptions(ANIMATION_OPTIONS[option])

    def apply_nice_numbers(self):
        """Adjust both axis to display nice round numbers."""
        self.axisX().applyNiceNumbers()
        self.axisY().applyNiceNumbers()

    def zoom_by_factor(self, factor: float):
        """Zoom in/out by factor (1.0 = no change).

        Make sure that we dont zoom out too far
        """
        self.zoom(factor)
        if self.axisX().min() < self.min_x:
            self.axisX().setMin(self.min_x)
        if self.axisX().max() > self.max_x:
            self.axisX().setMax(self.max_x)
        if self.axisY().max() > self.max_y:
            self.axisY().setMax(self.max_y)

        # always bottom-align when zooming for now. should perhaps become optional.
        # if self.axisY().min() < self.min_y:
        self.axisY().setMin(max(0, self.min_y))

    def get_chart_type(self) -> str:
        return CHART_TYPES.inv[self.chartType()]

    def get_margins(self) -> core.Margins:
        return core.Margins(self.margins())

    def get_plot_area(self) -> core.RectF:
        return core.RectF(self.plotArea())

    def get_locale(self) -> core.Locale:
        return core.Locale(self.locale())

    def get_theme(self) -> str:
        return THEMES.inv[self.theme()]

    def get_animation_options(self) -> List[str]:
        return [k for k, v in ANIMATION_OPTIONS.items() if v & self.animationOptions()]

    def get_animation_easing_curve(self) -> core.EasingCurve:
        return core.EasingCurve(self.animationEasingCurve())


if __name__ == "__main__":
    chart = Chart()
    legend = chart.get_legend()
    legend.set_alignment("bottom4")



# -*- coding: utf-8 -*-

from qtpy import QtCore


class RectF(QtCore.QRectF):
    def __repr__(self):
        return f"RectF({self.x()}, {self.y()}, {self.width()}, {self.height()})"


if __name__ == "__main__":
    rect = RectF(0, 2, 5, 5)
    print(repr(rect))



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import core, gui
from prettyqt.utils import bidict, InvalidParamError

ANCESTER_MODES = bidict(
    exclude_transients=QtGui.QWindow.ExcludeTransients,
    include_transients=QtGui.QWindow.IncludeTransients,
)

VISIBILITIES = bidict(
    windowed=QtGui.QWindow.Windowed,
    minimized=QtGui.QWindow.Minimized,
    maximized=QtGui.QWindow.Maximized,
    fullscreen=QtGui.QWindow.FullScreen,
    automatic=QtGui.QWindow.AutomaticVisibility,
    hidden=QtGui.QWindow.Hidden,
)

QtGui.QWindow.__bases__ = (core.Object, gui.Surface)


class Window(QtGui.QWindow):
    def __repr__(self):
        return f"{self.__class__.__name__}()"

    # def serialize_fields(self):
    #     return dict(
    #         speed=self.speed(),
    #         visibility=self.get_visibility(),
    #         scaled_size=self.scaledSize(),
    #         background_color=self.backgroundColor(),
    #     )

    def set_visibility(self, visibility: str):
        """Set window visibility.

        Valid values: "windowed", "maximized", "minimized", "fullscreen", "automatic",
                      "hidden"

        Args:
            visibility: window visibility

        Raises:
            InvalidParamError: window visibility does not exist
        """
        if visibility not in VISIBILITIES:
            raise InvalidParamError(visibility, VISIBILITIES)
        self.setVisibility(VISIBILITIES[visibility])

    def get_visibility(self) -> str:
        """Get the current window visibility.

        Possible values: "windowed", "maximized", "minimized", "fullscreen", "automatic",
                         "hidden"

        Returns:
            window visibility
        """
        return VISIBILITIES.inv[self.visibility()]


if __name__ == "__main__":
    wnd = Window()



# -*- coding: utf-8 -*-

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning


from prettyqt.utils import bidict

QGeoCoordinate = QtPositioning.QGeoCoordinate

COORDINATE_FORMATS = bidict(
    default=QGeoCoordinate.Degrees,
    with_hemisphere=QGeoCoordinate.DegreesWithHemisphere,
    min=QGeoCoordinate.DegreesMinutes,
    min_with_hemisphere=QGeoCoordinate.DegreesMinutesWithHemisphere,
    min_sec=QGeoCoordinate.DegreesMinutesSeconds,
    min_sec_with_hemisphere=QGeoCoordinate.DegreesMinutesSecondsWithHemisphere,
)

COORDINATE_TYPES = bidict(
    invalid=QtPositioning.QGeoCoordinate.InvalidCoordinate,
    two_d=QtPositioning.QGeoCoordinate.Coordinate2D,
    three_d=QtPositioning.QGeoCoordinate.Coordinate3D,
)


class GeoCoordinate(QtPositioning.QGeoCoordinate):
    def __str__(self):
        return self.toString()

    def __repr__(self):
        return f"GeoCoordinate({self.latitude()}, {self.longitude()})"

    def __bool__(self):
        return self.isValid()

    def get_type(self) -> str:
        return COORDINATE_TYPES.inv[self.type()]


if __name__ == "__main__":
    coord = GeoCoordinate(11, 11)
    print(str(coord))
    print(repr(coord))



# -*- coding: utf-8 -*-

from qtpy import QtLocation

from prettyqt import location


QtLocation.QPlaceResult.__bases__ = (location.PlaceSearchResult,)


class PlaceResult(QtLocation.QPlaceResult):
    def get_place(self) -> location.Place:
        return location.Place(self.place())



from qtpy import QtWidgets
from prettyqt import widgets


QtWidgets.QGraphicsLinearLayout.__bases__ = (widgets.GraphicsLayout,)


class GraphicsLinearLayout(QtWidgets.QGraphicsLinearLayout):
    def serialize_fields(self):
        return dict(items=self.get_children())

    def __setstate__(self, state):
        self.__init__()
        for i, (item, pos) in enumerate(zip(state["widgets"], state["positions"])):
            x, y = pos
            self[x, y] = item

    def __add__(self, other):
        self[self.count()] = other
        return self


if __name__ == "__main__":
    app = widgets.app()
    layout = GraphicsLinearLayout()



# -*- coding: utf-8 -*-

import pathlib
from typing import Optional, Union, Iterator, MutableMapping, Mapping
import logging

from qtpy import QtCore, QtWidgets

from prettyqt import core, gui, widgets
from prettyqt.utils import bidict, colors, InvalidParamError

logger = logging.getLogger(__name__)

STANDARD_PIXMAPS = widgets.style.STANDARD_PIXMAPS

QtWidgets.QApplication.__bases__ = (gui.GuiApplication,)

UI_EFFECTS = bidict(
    animate_menu=QtCore.Qt.UI_AnimateMenu,
    fade_menu=QtCore.Qt.UI_FadeMenu,
    animate_combo=QtCore.Qt.UI_AnimateCombo,
    animate_tooltip=QtCore.Qt.UI_AnimateTooltip,
    fade_tooltip=QtCore.Qt.UI_FadeTooltip,
    animate_toolbox=QtCore.Qt.UI_AnimateToolBox,
)

NAVIGATION_MODES = bidict(
    none=QtCore.Qt.NavigationModeNone,
    keypad_tab_order=QtCore.Qt.NavigationModeKeypadTabOrder,
    keypad_directional=QtCore.Qt.NavigationModeKeypadDirectional,
    cursor_auto=QtCore.Qt.NavigationModeCursorAuto,
    cursor_force_visible=QtCore.Qt.NavigationModeCursorForceVisible,
)


class Application(QtWidgets.QApplication):
    def __class_getitem__(cls, name: str) -> QtWidgets.QWidget:
        widget = cls.get_widget(name)
        if widget is None:
            raise ValueError(f"Widget {name!r} does not exist.")
        return widget

    def __iter__(self) -> Iterator[QtWidgets.QWidget]:
        return iter(self.topLevelWidgets())

    def serialize_fields(self):
        return dict(
            auto_sip_enabled=self.autoSipEnabled(),
            cursor_flash_time=self.cursorFlashTime(),
            double_click_interval=self.doubleClickInterval(),
            keyboard_input_interval=self.keyboardInputInterval(),
            start_drag_distance=self.startDragDistance(),
            start_drag_time=self.startDragTime(),
            style_sheet=self.styleSheet(),
            wheel_scroll_lines=self.wheelScrollLines(),
            icon=gui.Icon(self.windowIcon()),
        )

    def store_widget_states(
        self, settings: Optional[MutableMapping] = None, key: str = "states"
    ):
        splitters = self.find_children(QtWidgets.QSplitter)
        splitter_dct = {
            sp.objectName(): sp.saveState() for sp in splitters if sp.objectName()
        }
        mainwindows = self.find_children(QtWidgets.QMainWindow)
        mw_dct = {
            mw.objectName(): mw.saveState() for mw in mainwindows if mw.objectName()
        }
        headerviews = self.find_children(QtWidgets.QHeaderView)
        headerview_dct = {
            h.objectName(): h.saveState() for h in headerviews if h.objectName()
        }
        settings = core.Settings() if settings is None else settings
        settings[key] = dict(
            splitters=splitter_dct, mainwindows=mw_dct, headerviews=headerview_dct
        )

    def restore_widget_states(
        self, settings: Optional[Mapping] = None, key: str = "states"
    ):
        settings = core.Settings() if settings is None else settings
        splitters = settings[key].get("splitters")
        mainwindows = settings[key].get("splitters")
        headerviews = settings[key].get("headerviews")
        if splitters is not None:
            for k, v in splitters.items():
                w = self.find_child(QtWidgets.QSplitter, name=k)
                if w is not None:
                    w.restoreState(v)
        if mainwindows is not None:
            for k, v in mainwindows.items():
                w = self.find_child(QtWidgets.QMainWindow, name=k)
                if w is not None:
                    w.restoreState(v)
        if headerviews is not None:
            for k, v in headerviews.items():
                w = self.find_child(QtWidgets.QHeaderView, name=k)
                if w is not None:
                    w.restoreState(v)

    def set_icon(self, icon: gui.icon.IconType):
        """Set the default window icon.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon, color=colors.WINDOW_ICON_COLOR)
        self.setWindowIcon(icon)

    def load_language_file(self, file: Union[pathlib.Path, str]):
        translator = core.Translator(self)
        if file in ["de", "fr"]:
            path = pathlib.Path(__file__).parent.parent
            file = path / "localization" / f"qtbase_{file}.ts"
        translator.load(str(file))
        self.installTranslator(translator)

    def set_metadata(
        self,
        app_name: Optional[str] = None,
        app_version: Optional[str] = None,
        org_name: Optional[str] = None,
        org_domain: Optional[str] = None,
    ):
        if app_name is not None:
            self.setApplicationName(app_name)
        if app_version is not None:
            self.setApplicationVersion(app_name)
        if org_name is not None:
            self.setOrganizationName(org_name)
        if org_domain is not None:
            self.setOrganizationDomain(org_domain)
        # import ctypes
        # myappid = 'mycompany.myproduct.subproduct.version' # arbitrary string
        # ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

    def about_popup(self, title: str = "About"):
        text = (
            f"{self.applicationName()}\n\n"
            f"{self.organizationName()}\n"
            f"{self.applicationVersion()}\n"
            f"{self.organizationDomain()}"
        )
        popup = widgets.MessageBox(
            widgets.MessageBox.NoIcon, title, text, buttons=widgets.MessageBox.Ok
        )
        popup.set_icon("mdi.information-outline")
        popup.exec_()

    def main_loop(self) -> int:
        return self.exec_()

    @classmethod
    def get_font(cls) -> gui.Font:
        return gui.Font(cls.font())

    @classmethod
    def use_hdpi_bitmaps(cls, state: bool = True):
        cls.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, state)

    @classmethod
    def disable_window_help_button(cls, state: bool = True):
        cls.setAttribute(QtCore.Qt.AA_DisableWindowContextHelpButton, state)

    @classmethod
    def copy_to_clipboard(cls, text: str):
        """Sets clipboard to supplied text."""
        cb = cls.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText(text, mode=cb.Clipboard)

    @classmethod
    def get_mainwindow(cls) -> Optional[QtWidgets.QMainWindow]:
        widget_list = cls.instance().topLevelWidgets()
        for widget in widget_list:
            if isinstance(widget, QtWidgets.QMainWindow):
                return widget
        return None

    @classmethod
    def get_widget(cls, name: str) -> Optional[QtWidgets.QWidget]:
        mw = cls.get_mainwindow()
        if mw is None:
            logger.warning("Trying to get widget from nonexistent mainwindow")
            return None
        return mw.findChild(QtWidgets.QWidget, name)
        # widget_list = cls.instance().allWidgets()
        # for widget in widget_list:
        #     if isinstance(widget, QtWidgets.QWidget) and widget.objectName() == name:
        #         return widget
        # return None

    @classmethod
    def get_icon(cls, icon: str) -> gui.Icon:
        style = cls.style()
        # icon_size = style.pixelMetric(QtWidgets.QStyle.PM_MessageBoxIconSize)
        if icon not in STANDARD_PIXMAPS:
            raise InvalidParamError(icon, STANDARD_PIXMAPS)
        icon = style.standardIcon(STANDARD_PIXMAPS[icon])
        return gui.Icon(icon)

    def set_effect_enabled(self, effect: str, enabled: bool = True):
        """Set the enabled state of a desktop effect.

        valid values are: "animate_menu", "fade_menu", "animate_combo",
        "animate_tooltip", "fade_tooltip", "animate_toolbox"

        Args:
            effect: desktop effect to set
            enabled: new state

        Raises:
            InvalidParamError: invalid desktop effect
        """
        if effect not in UI_EFFECTS:
            raise InvalidParamError(effect, UI_EFFECTS)
        self.setEffectEnabled(UI_EFFECTS[effect])

    def is_effect_enabled(self, effect: str) -> str:
        """Return desktop effect state.

        possible values are "animate_menu", "fade_menu", "animate_combo",
        "animate_tooltip", "fade_tooltip", "animate_toolbox"

        Returns:
            desktop effect state
        """
        return self.isEffectEnabled(UI_EFFECTS[effect])

    def set_navigation_mode(self, mode: str):
        """Set the navigation mode.

        valid values: "none", "keypad_tab_order", "keypad_directional", "cursor_auto",
        "cursor_force_visible"

        Args:
            mode: navigation mode to use

        Raises:
            InvalidParamError: invalid navigation mode
        """
        if mode not in NAVIGATION_MODES:
            raise InvalidParamError(mode, NAVIGATION_MODES)
        self.setNavigationMode(NAVIGATION_MODES[mode])

    def get_navigation_mode(self) -> str:
        """Return navigation mode.

        possible values: "none", "keypad_tab_order", "keypad_directional", "cursor_auto",
        "cursor_force_visible"

        Returns:
            navigation mode
        """
        return NAVIGATION_MODES.inv[self.navigationMode()]


if __name__ == "__main__":
    app = Application([])
    app.load_language_file("de")



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import QtQml, QtCore

from prettyqt import core
from prettyqt.utils import bidict

COMPILATION_MODES = bidict(
    prefer_synchronous=QtQml.QQmlComponent.PreferSynchronous,
    asynchronous=QtQml.QQmlComponent.Asynchronous,
)

STATUS = bidict(
    null=QtQml.QQmlComponent.Null,
    ready=QtQml.QQmlComponent.Ready,
    loading=QtQml.QQmlComponent.Loading,
    error=QtQml.QQmlComponent.Error,
)

QtQml.QQmlComponent.__bases__ = (core.Object,)


class QmlComponent(QtQml.QQmlComponent):
    def get_status(self) -> str:
        return STATUS.inv[self.status()]

    def get_url(self) -> core.Url:
        return core.Url(self.url())

    def load_url(self, url: Union[QtCore.QUrl, str], mode: str):
        if isinstance(url, str):
            url = core.Url.from_user_input(url)
        self.loadUrl(url, COMPILATION_MODES[mode])



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError

CACHE_MODES = bidict(none=QtGui.QMovie.CacheNone, all=QtGui.QMovie.CacheAll)

MOVIE_STATES = bidict(
    not_running=QtGui.QMovie.NotRunning,
    paused=QtGui.QMovie.Paused,
    running=QtGui.QMovie.Running,
)

QtGui.QMovie.__bases__ = (core.Object,)


class Movie(QtGui.QMovie):
    def __repr__(self):
        return f"{self.__class__.__name__}()"

    def serialize_fields(self):
        return dict(
            speed=self.speed(),
            cache_mode=self.get_cache_mode(),
            scaled_size=self.scaledSize(),
            background_color=self.backgroundColor(),
        )

    def set_cache_mode(self, mode: str):
        """Set cache mode.

        Valid values for cache_mode: "none", "all"

        Args:
            mode: cache mode

        Raises:
            InvalidParamError: cache mode does not exist
        """
        if mode not in CACHE_MODES:
            raise InvalidParamError(mode, CACHE_MODES)
        self.setCacheMode(CACHE_MODES[mode])

    def get_cache_mode(self) -> str:
        """Get the current cache mode.

        Possible values: "none", "all"

        Returns:
            cache mode
        """
        return CACHE_MODES.inv[self.cacheMode()]


if __name__ == "__main__":
    movie = Movie()



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import QtMultimedia

from prettyqt.utils import bidict

HANDLE_TYPES = bidict(
    none=QtMultimedia.QAbstractVideoBuffer.NoHandle,
    gl_texture=QtMultimedia.QAbstractVideoBuffer.GLTextureHandle,
    xv_shm_image=QtMultimedia.QAbstractVideoBuffer.XvShmImageHandle,
    core_image=QtMultimedia.QAbstractVideoBuffer.CoreImageHandle,
    pixmap=QtMultimedia.QAbstractVideoBuffer.QPixmapHandle,
    egl_image=QtMultimedia.QAbstractVideoBuffer.EGLImageHandle,
    user=QtMultimedia.QAbstractVideoBuffer.UserHandle,
)

MAP_MODE = bidict(
    not_mapped=QtMultimedia.QAbstractVideoBuffer.NotMapped,
    read_only=QtMultimedia.QAbstractVideoBuffer.ReadOnly,
    write_only=QtMultimedia.QAbstractVideoBuffer.WriteOnly,
    read_write=QtMultimedia.QAbstractVideoBuffer.ReadWrite,
)


class AbstractVideoBuffer(QtMultimedia.QAbstractVideoBuffer):
    def __init__(self, handle_type: Union[int, str]):
        if handle_type in HANDLE_TYPES:
            handle_type = HANDLE_TYPES[handle_type]
        super().__init__(handle_type)

    def __repr__(self):
        return f"{type(self).__name__}({self.get_handle_type()!r})"

    def get_handle_type(self) -> str:
        """Return current handle type.

        Possible values: "none", "gl_texture", "xv_shm_image", "core_image", "pixmap",
                         "egl_image", "user"

        Returns:
            handle type
        """
        return HANDLE_TYPES.inv[self.handleType()]

    def get_map_mode(self) -> str:
        """Return current map mode.

        Possible values: "not_mapped", "read_only", "write_only", "read_write"

        Returns:
            map mode
        """
        return MAP_MODE.inv[self.mapMode()]

    def map_planes(
        self,
        mode: Union[int, str],
        num_bytes: int,
        bytes_per_line: int = 4,
        data: int = 4,
    ):
        if mode in MAP_MODE:
            mode = MAP_MODE[mode]
        self.mapPlanes(mode, num_bytes, bytes_per_line, data)

    def map(
        self,
        mode: Union[int, str],
        num_bytes: int,
        bytes_per_line: int = 4,
    ):
        if mode in MAP_MODE:
            mode = MAP_MODE[mode]
        super().map(mode, num_bytes, bytes_per_line)



import math

from qtpy import QtCore

from prettyqt import core


class Line(QtCore.QLine):
    def __repr__(self):
        return f"Line({repr(self.get_p1())}, {repr(self.get_p2())})"

    def __abs__(self) -> float:
        p = self.get_p2() - self.get_p1()
        return math.sqrt((p.x() * p.x()) + (p.y() * p.y()))

    def __reversed__(self):
        self[0], self[1] = self[1], self[0]
        return self

    def __iter__(self):
        yield self.get_p1()
        yield self.get_p2()

    def __getitem__(self, index: int) -> core.Point:
        if index == 0:
            return self.get_p1()
        elif index == 1:
            return self.get_p2()

    def __setitem__(self, index: int, value: QtCore.QPoint):
        if index == 0:
            self.setP1(value)
        elif index == 1:
            self.setP2(value)

    def get_p1(self) -> core.Point:
        return core.Point(self.p1())

    def get_p2(self) -> core.Point:
        return core.Point(self.p2())


if __name__ == "__main__":
    line = Line(core.Point(0, 0), core.Point(2, 2))
    print(reversed(line))



# -*- coding: utf-8 -*-

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineWidgets

# from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError

Item = QtWebEngineWidgets.QWebEngineDownloadItem

DOWNLOAD_INTERRUPT_REASONS = bidict(
    none=Item.NoReason,
    file_failed=Item.FileFailed,
    file_access_denied=Item.FileAccessDenied,
    file_no_space=Item.FileNoSpace,
    filename_too_long=Item.FileNameTooLong,
    file_too_large=Item.FileTooLarge,
    file_virus_infected=Item.FileVirusInfected,
    file_transient_error=Item.FileTransientError,
    file_blocked=Item.FileBlocked,
    file_security_check_failed=Item.FileSecurityCheckFailed,
    file_too_short=Item.FileTooShort,
    file_hash_mismatch=Item.FileHashMismatch,
    network_fialed=Item.NetworkFailed,
    network_timeout=Item.NetworkTimeout,
    network_disconnected=Item.NetworkDisconnected,
    network_server_down=Item.NetworkServerDown,
    network_invalid_request=Item.NetworkInvalidRequest,
    server_failed=Item.ServerFailed,
    server_bad_content=Item.ServerBadContent,
    server_unauthorized=Item.ServerUnauthorized,
    server_cert_problem=Item.ServerCertProblem,
    server_forbidden=Item.ServerForbidden,
    server_unreachable=Item.ServerUnreachable,
    user_canceled=Item.UserCanceled,
)

DOWNLOAD_STATES = bidict(
    requested=QtWebEngineWidgets.QWebEngineDownloadItem.DownloadRequested,
    in_progress=QtWebEngineWidgets.QWebEngineDownloadItem.DownloadInProgress,
    completed=QtWebEngineWidgets.QWebEngineDownloadItem.DownloadCompleted,
    cancelled=QtWebEngineWidgets.QWebEngineDownloadItem.DownloadCancelled,
    interrupted=QtWebEngineWidgets.QWebEngineDownloadItem.DownloadInterrupted,
)

SAVE_PAGE_FORMATS = bidict(
    unknown=QtWebEngineWidgets.QWebEngineDownloadItem.UnknownSaveFormat,
    single_html=QtWebEngineWidgets.QWebEngineDownloadItem.SingleHtmlSaveFormat,
    complete_html=QtWebEngineWidgets.QWebEngineDownloadItem.CompleteHtmlSaveFormat,
    mime_html=QtWebEngineWidgets.QWebEngineDownloadItem.MimeHtmlSaveFormat,
)

# QtWebEngineWidgets.QWebEngineDownloadItem.__bases__ = (core.Object,)


class WebEngineDownloadItem(object):
    def __init__(self, item: QtWebEngineWidgets.QWebEngineDownloadItem):
        self.item = item

    def __getattr__(self, val):
        return getattr(self.item, val)

    def get_interrupt_reason(self) -> str:
        return DOWNLOAD_INTERRUPT_REASONS.inv[self.item.interruptReason()]

    def get_state(self) -> str:
        return DOWNLOAD_STATES.inv[self.item.state()]

    def set_save_page_format(self, fmt: str):
        """Set the save page format.

        Allowed values are "unknown", "single_html", "complete_html", "mime_html"

        Args:
            fmt: save page format for the layout

        Raises:
            InvalidParamError: save page format does not exist
        """
        if fmt not in SAVE_PAGE_FORMATS:
            raise InvalidParamError(fmt, SAVE_PAGE_FORMATS)
        self.item.setSavePageFormat(SAVE_PAGE_FORMATS[fmt])

    def get_save_page_format(self) -> str:
        """Return current save page format.

        Possible values are "unknown", "single_html", "complete_html", "mime_html"

        Returns:
            Save page format
        """
        return SAVE_PAGE_FORMATS.inv[self.item.savePageFormat()]


# if __name__ == "__main__":
#     from prettyqt import widgets

# app = widgets.app()
# item = WebEngineDownloadItem()
# app.main_loop()



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtQml

from prettyqt import core, qml
from prettyqt.utils import bidict

EXTENSIONS = bidict(
    translation=QtQml.QJSEngine.TranslationExtension,
    console=QtQml.QJSEngine.ConsoleExtension,
    garbage_collection=QtQml.QJSEngine.GarbageCollectionExtension,
    all=QtQml.QJSEngine.AllExtensions,
)

QtQml.QJSEngine.__bases__ = (core.Object,)


class JSEngine(QtQml.QJSEngine):
    def serialize_fields(self):
        return dict(ui_language=self.uiLanguage())

    def install_extensions(self, extension: str, obj: Optional[QtQml.QJSValue] = None):
        if obj is None:
            obj = QtQml.QJSValue()
        self.installExtensions(EXTENSIONS[extension], obj)

    def new_array(self, length=0) -> qml.JSValue:
        return qml.JSValue(self.newArray(length))


if __name__ == "__main__":
    app = core.CoreApplication([])
    val = JSEngine()
    arr = val.new_array()
    arr["test"] = 1
    arr["tes2t"] = 1
    assert arr["test"] == 1
    for name, val in arr:
        print(name, val)



# -*- coding: utf-8 -*-

from __future__ import annotations

from typing import Tuple, Iterator

from qtpy.QtCharts import QtCharts

from prettyqt import charts
from prettyqt.utils import bidict, InvalidParamError

LABELS_POSITIONS = bidict(
    center=QtCharts.QCategoryAxis.AxisLabelsPositionCenter,
    on_value=QtCharts.QCategoryAxis.AxisLabelsPositionOnValue,
)

QtCharts.QCategoryAxis.__bases__ = (charts.ValueAxis,)


class CategoryAxis(QtCharts.QCategoryAxis):
    def __delitem__(self, index: str):
        self.remove(index)

    def __getitem__(self, label: str) -> str:
        return self.categoriesLabels()[label]

    def __setitem__(self, index: str, value: str):
        self.replaceLabel(index, value)

    def __iter__(self) -> Iterator[str]:
        return iter(self.categoriesLabels())

    def __add__(self, other: Tuple[str, int]) -> CategoryAxis:
        self.append(*other)
        return self

    def set_labels_position(self, position: str):
        """Set the labels position.

        Allowed values are "center", "on_value"

        Args:
            position: labels position

        Raises:
            InvalidParamError: labels position does not exist
        """
        if position not in LABELS_POSITIONS:
            raise InvalidParamError(position, LABELS_POSITIONS)
        self.setLabelsPosition(LABELS_POSITIONS[position])

    def get_labels_position(self) -> str:
        """Return current labels position.

        Possible values: "center", "on_value"

        Returns:
            labels position
        """
        return LABELS_POSITIONS.inv[self.labelsPosition()]


if __name__ == "__main__":
    axis = CategoryAxis()



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui


QtGui.QRasterWindow.__bases__ = (gui.PaintDeviceWindow,)


class RasterWindow(QtGui.QRasterWindow):
    pass


if __name__ == "__main__":
    app = gui.GuiApplication([])



# -*- coding: utf-8 -*-

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning


from prettyqt.utils import bidict, InvalidParamError

SATELLITE_SYSTEMS = bidict(
    undefined=QtPositioning.QGeoSatelliteInfo.Undefined,
    gps=QtPositioning.QGeoSatelliteInfo.GPS,
    glonass=QtPositioning.QGeoSatelliteInfo.GLONASS,
)

ATTRIBUTES = bidict(
    elevation=QtPositioning.QGeoSatelliteInfo.Elevation,
    azimuth=QtPositioning.QGeoSatelliteInfo.Azimuth,
)


class GeoSatelliteInfo(QtPositioning.QGeoSatelliteInfo):
    def __getitem__(self, index: str):
        return self.attribute(ATTRIBUTES[index])

    def __setitem__(self, index: str, value: float):
        self.setAttribute(ATTRIBUTES[index], value)

    def __delitem__(self, index: str):
        self.removeAttribute(ATTRIBUTES[index])

    def __contains__(self, value: str):
        return self.hasAttribute(ATTRIBUTES[value])

    def __int__(self):
        return self.satelliteIdentifier()

    def set_satellite_system(self, system: str):
        """Set satellite system.

        valid values are: "undefined", "gps", "glonass"

        Args:
            system: satellite system to use

        Raises:
            InvalidParamError: invalid system
        """
        if system not in SATELLITE_SYSTEMS:
            raise InvalidParamError(system, SATELLITE_SYSTEMS)
        self.setSatelliteSystem(SATELLITE_SYSTEMS[system])

    def get_satellite_system(self) -> bool:
        """Return satellite system.

        possible values are "undefined", "gps", "glonass"

        Returns:
            satellite system
        """
        return SATELLITE_SYSTEMS.inv[self.satelliteSystem()]


if __name__ == "__main__":
    coord = GeoSatelliteInfo()



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QGraphicsProxyWidget.__bases__ = (widgets.GraphicsWidget,)


class GraphicsProxyWidget(QtWidgets.QGraphicsProxyWidget):

    pass



# -*- coding: utf-8 -*-

"""TextToSpeech module.

Contains QtTextToSpeech-based classes
"""

from .voice import Voice
from .texttospeech import TextToSpeech

__all__ = [
    "Voice",
    "TextToSpeech",
]



# -*- coding: utf-8 -*-

from typing import Union, Iterable, Mapping, List, Any, Optional, Iterator

from qtpy import QtCore, QtWidgets

from prettyqt import core, gui, widgets
from prettyqt.utils import InvalidParamError


QtWidgets.QListWidget.__bases__ = (widgets.ListView,)


class NoData(object):
    pass


class ListWidget(QtWidgets.QListWidget):

    value_changed = core.Signal(object)

    def __init__(
        self, parent: Optional[QtWidgets.QWidget] = None, selection_mode: str = "single"
    ):
        super().__init__(parent)
        self.itemSelectionChanged.connect(self.on_index_change)
        self.set_selection_mode(selection_mode)

    def __repr__(self):
        return f"ListWidget: {self.count()} items"

    def __getitem__(self, row: int) -> QtWidgets.QListWidgetItem:
        return self.item(row)

    def __delitem__(self, row: int):
        self.takeItem(row)

    def __add__(self, other: QtWidgets.QListWidgetItem):
        self.addItem(other)
        return self

    def __iter__(self) -> Iterator[QtWidgets.QListWidgetItem]:
        return iter(self.get_children())

    def __len__(self) -> int:
        return self.count()

    def serialize_fields(self):
        return dict(
            items=self.get_children(),
            selection_mode=self.get_selection_mode(),
            sorting_enabled=self.isSortingEnabled(),
            current_row=self.currentRow(),
        )

    def __setstate__(self, state):
        self.__init__()
        self.set_selection_mode(state["selection_mode"])
        self.setSortingEnabled(state["sorting_enabled"])
        self.setCurrentRow(state["current_row"])
        for item in state["items"]:
            self.addItem(item)

    def sort(self, reverse: bool = False):
        order = QtCore.Qt.DescendingOrder if reverse else QtCore.Qt.AscendingOrder
        self.sortItems(order)

    def on_index_change(self):
        data = self.get_value()
        self.value_changed.emit(data)

    def get_children(self) -> List[QtWidgets.QListWidgetItem]:
        return [self.item(row) for row in range(self.count())]

    def add_items(self, items: Union[Iterable, Mapping]):
        if isinstance(items, Mapping):
            for k, v in items.items():
                self.add(v, k)
        else:
            for i in items:
                if isinstance(i, (tuple, list)):
                    self.add(*i)
                else:
                    self.add(i)

    def add(self, label: str, data=NoData, icon: gui.icon.IconType = None):
        if data is NoData:
            data = label
        item = widgets.ListWidgetItem(label)
        item.set_icon(icon)
        item.setData(QtCore.Qt.UserRole, data)
        self.addItem(item)

    def get_value(self) -> List[Any]:
        return [i.data(QtCore.Qt.UserRole) for i in self.selectedItems()]

    def set_value(self, value):
        for i in self.get_children():
            if i.data(QtCore.Qt.UserRole) in value:
                self.setCurrentItem(i)
                break

    def scroll_to_item(
        self, item: QtWidgets.QListWidgetItem, mode: str = "ensure_visible"
    ):
        if mode not in widgets.abstractitemview.SCROLL_HINTS:
            raise InvalidParamError(mode, widgets.abstractitemview.SCROLL_HINTS)
        self.scrollToItem(item, widgets.abstractitemview.SCROLL_HINTS[mode])


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = ListWidget()
    widget.add("test", icon="mdi.timer")
    widget.add("test", icon="mdi.timer")
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import core, gui


QtGui.QRegExpValidator.__bases__ = (gui.Validator,)


class RegExpValidator(QtGui.QRegExpValidator):
    def __repr__(self):
        return f"RegExpValidator(RegExp({self.get_regex()!r}))"

    def __getstate__(self):
        return dict(regexp=core.RegExp(self.regExp()))

    def __setstate__(self, state):
        self.__init__()
        self.setRegExp(state["regexp"])

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return False
        return self.regExp() == other.regExp()

    def set_regex(self, regex: str):
        re = core.RegExp(regex)
        self.setRegExp(re)

    def get_regex(self) -> str:
        val = self.regExp()
        return val.pattern()


if __name__ == "__main__":
    val = RegExpValidator()
    val.set_regex(r"\w\d\d")



# -*- coding: utf-8 -*-

from typing import Union, Optional

from qtpy import QtLocation


class PlaceAttribute(QtLocation.QPlaceAttribute):
    def __init__(
        self,
        other_or_label: Union[None, str, QtLocation.QPlaceAttribute] = None,
        value: Optional[str] = None,
    ):
        if isinstance(other_or_label, QtLocation.QPlaceAttribute):
            super().__init__(other_or_label)
        else:
            super().__init__()
            self.setLabel(other_or_label)
            self.setText(value)

    def __repr__(self):
        return f"{type(self).__name__}({self.label()!r}, {self.text()!r})"

    def __str__(self):
        return f"{self.label()}: {self.text()}"

    def __bool__(self):
        return not self.isEmpty()


if __name__ == "__main__":
    attr = PlaceAttribute("test", "us")
    print(repr(attr))



from qtpy import QtWidgets

from prettyqt import core, widgets


class CollapsibleFrame(widgets.Frame):

    expanded = core.Signal()
    collapsed = core.Signal()

    def __init__(self, parent: QtWidgets.QWidget = None):
        super().__init__(parent)
        self.set_frame_shape("styled_panel")
        self.set_frame_shadow("plain")
        # layout
        self._layout = widgets.BoxLayout("vertical")
        self._layout.set_margin(0)
        self._layout.setSpacing(0)
        self.setLayout(self._layout)
        # button
        self._button = widgets.ToolButton(self)
        self._button.set_arrow_type("right")
        self._button.set_style("text_beside_icon")
        self._button.setAutoRaise(False)
        self._button.set_text("CollapsibleFrame")
        self.set_size_policy("minimum_expanding", "fixed")
        self._layout.addWidget(self._button, 0)
        self._button.setVisible(True)
        # group box
        self._panel = widgets.Widget(self)
        self._layout.addWidget(self._panel)
        self._panel.setVisible(False)
        self._panel_layout = widgets.BoxLayout("vertical")
        self._panel_layout.set_margin(1)
        self._panel_layout.setSpacing(2)
        self._panel.setLayout(self._panel_layout)
        # connect signals
        self._button.clicked.connect(self.on_button_click)
        # private state variables
        self._is_collapsed = True

    def set_title(self, title: str):
        self._button.set_text(title)

    def add_widget(self, widget: widgets.Widget):
        self._panel_layout.addWidget(widget)

    def remove_widget(self, widget: widgets.Widget):
        self._panel_layout.removeWidget(widget)

    def is_expanded(self) -> bool:
        return not self._is_collapsed

    def expand(self):
        self._button.set_arrow_type("down")
        self._panel.setVisible(True)
        self._is_collapsed = False
        self.set_size_policy("minimum_expanding", "minimum_expanding")

    def collapse(self):
        self._panel.setVisible(False)
        self._button.set_arrow_type("right")
        self._is_collapsed = True
        self.set_size_policy("preferred", "preferred")

    @core.Slot()
    def on_button_click(self):
        if self._is_collapsed:
            self.expand()
            self.expanded.emit()
            return
        else:
            self.collapse()
            self.collapsed.emit()
            return


if __name__ == "__main__":
    app = widgets.app()
    widget = CollapsibleFrame()
    widget.add_widget(widgets.Label("test"))
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from typing import Any, Dict
from qtpy import QtWidgets

from prettyqt.utils import bidict, helpers, prettyprinter


SIZE_POLICIES = bidict(
    fixed=QtWidgets.QSizePolicy.Fixed,
    minimum=QtWidgets.QSizePolicy.Minimum,
    maximum=QtWidgets.QSizePolicy.Maximum,
    preferred=QtWidgets.QSizePolicy.Preferred,
    expanding=QtWidgets.QSizePolicy.Expanding,
    minimum_expanding=QtWidgets.QSizePolicy.MinimumExpanding,
    ignored=QtWidgets.QSizePolicy.Ignored,
)

CONTROL_TYPES = bidict(
    default=QtWidgets.QSizePolicy.DefaultType,
    buttonbox=QtWidgets.QSizePolicy.ButtonBox,
    checkbox=QtWidgets.QSizePolicy.CheckBox,
    combobox=QtWidgets.QSizePolicy.ComboBox,
    frame=QtWidgets.QSizePolicy.Frame,
    groupbox=QtWidgets.QSizePolicy.GroupBox,
    label=QtWidgets.QSizePolicy.Label,
    line=QtWidgets.QSizePolicy.Line,
    lineedit=QtWidgets.QSizePolicy.LineEdit,
    pushbutton=QtWidgets.QSizePolicy.PushButton,
    radiobutton=QtWidgets.QSizePolicy.RadioButton,
    slider=QtWidgets.QSizePolicy.Slider,
    spinbox=QtWidgets.QSizePolicy.SpinBox,
    tabwidget=QtWidgets.QSizePolicy.TabWidget,
    toolbutton=QtWidgets.QSizePolicy.ToolButton,
)


class SizePolicy(prettyprinter.PrettyPrinter, QtWidgets.QSizePolicy):
    def __repr__(self) -> str:
        cls_name = self.__class__.__name__
        params = helpers.format_kwargs(self.__getstate__())
        return f"{cls_name}({params})"

    def __getstate__(self):
        return dict(
            has_height_for_width=self.hasHeightForWidth(),
            has_width_for_height=self.hasWidthForHeight(),
            horizontal_stretch=self.horizontalStretch(),
            vertical_stretch=self.verticalStretch(),
            horizontal_policy=self.get_horizontal_policy(),
            vertical_policy=self.get_vertical_policy(),
            retain_size_when_hidden=self.retainSizeWhenHidden(),
        )

    def __setstate__(self, state: Dict[str, Any]) -> None:
        super().__init__()
        self.setHeightForWidth(state["has_height_for_width"])
        self.setWidthForHeight(state["has_width_for_height"])
        self.setHorizontalStretch(state["horizontal_stretch"])
        self.setVerticalStretch(state["vertical_stretch"])
        self.set_horizontal_policy(state["horizontal_policy"])
        self.set_vertical_policy(state["vertical_policy"])
        self.setRetainSizeWhenHidden(state["retain_size_when_hidden"])

    def serialize(self):
        return self.__getstate__()

    def get_horizontal_policy(self) -> str:
        """Return size policy.

        possible values are "fixed", "minimum", "maximum", "preferred",
        "expanding", "minimum_expanding" and "ignored"

        Returns:
            horizontal size policy
        """
        return SIZE_POLICIES.inv[self.horizontalPolicy()]

    def set_horizontal_policy(self, mode: str) -> None:
        """Set the horizontal policy.

        possible values are "fixed", "minimum", "maximum", "preferred",
        "expanding", "minimum_expanding" and "ignored"

        Args:
            mode: policy to set
        """
        self.setHorizontalPolicy(SIZE_POLICIES[mode])

    def get_vertical_policy(self) -> str:
        """Return size policy.

        possible values are "fixed", "minimum", "maximum", "preferred",
        "expanding", "minimum_expanding" and "ignored"

        Returns:
            vertical size policy
        """
        return SIZE_POLICIES.inv[self.verticalPolicy()]

    def set_vertical_policy(self, mode: str) -> None:
        """Set the horizontal policy.

        possible values are "fixed", "minimum", "maximum", "preferred",
        "expanding", "minimum_expanding" and "ignored"

        Args:
            mode: policy to set
        """
        self.setVerticalPolicy(SIZE_POLICIES[mode])

    def get_control_type(self) -> str:
        """Return control type.

        possible values are "default", "buttonbox", "checkbox", "combobox", "frame",
        "groupbox", "label", "line", "lineedit", "pushbutton", "radiobutton", "slider",
        "spinbox", "tabwidget", "toolbutton"

        Returns:
            control type
        """
        return CONTROL_TYPES.inv[self.controlType()]

    def set_control_type(self, mode: str) -> None:
        """Set the control type.

        possible values are "default", "buttonbox", "checkbox", "combobox", "frame",
        "groupbox", "label", "line", "lineedit", "pushbutton", "radiobutton", "slider",
        "spinbox", "tabwidget", "toolbutton"

        Args:
            mode: control type to set
        """
        self.setControlType(CONTROL_TYPES[mode])



# -*- coding: utf-8 -*-

from qtpy import QtMultimedia

from prettyqt import multimedia
from prettyqt.utils import bidict


STATES = bidict(
    stopped=QtMultimedia.QMediaPlayer.StoppedState,
    playing=QtMultimedia.QMediaPlayer.PlayingState,
    paused=QtMultimedia.QMediaPlayer.PausedState,
)

ERRORS = bidict(
    none=QtMultimedia.QMediaPlayer.NoError,
    resource=QtMultimedia.QMediaPlayer.ResourceError,
    format=QtMultimedia.QMediaPlayer.FormatError,
    network=QtMultimedia.QMediaPlayer.NetworkError,
    access_denies=QtMultimedia.QMediaPlayer.AccessDeniedError,
    service_missing=QtMultimedia.QMediaPlayer.ServiceMissingError,
)

MEDIA_STATUS = bidict(
    unknown=QtMultimedia.QMediaPlayer.UnknownMediaStatus,
    no_media=QtMultimedia.QMediaPlayer.NoMedia,
    loading=QtMultimedia.QMediaPlayer.LoadingMedia,
    loaded=QtMultimedia.QMediaPlayer.LoadedMedia,
    stalled=QtMultimedia.QMediaPlayer.StalledMedia,
    buffering=QtMultimedia.QMediaPlayer.BufferingMedia,
    buffered=QtMultimedia.QMediaPlayer.BufferedMedia,
    end=QtMultimedia.QMediaPlayer.EndOfMedia,
    invalid=QtMultimedia.QMediaPlayer.InvalidMedia,
)

QtMultimedia.QMediaPlayer.__bases__ = (multimedia.MediaObject,)


class MediaPlayer(QtMultimedia.QMediaPlayer):
    def get_state(self) -> str:
        """Return current state.

        Possible values: "stopped", "playing", "paused"

        Returns:
            state
        """
        return STATES.inv[self.state()]

    def get_media_status(self) -> str:
        """Return current media status.

        Possible values: "unknown", "no_media", "loading", "loaded", "stalled",
                         "buffering", "buffered", "end", "invalid"

        Returns:
            media status
        """
        return MEDIA_STATUS.inv[self.mediaStatus()]



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import QtWidgets, QtGui

from prettyqt import core, gui, widgets
from prettyqt.utils import InvalidParamError

STANDARD_PIXMAPS = widgets.style.STANDARD_PIXMAPS

QtWidgets.QAbstractButton.__bases__ = (widgets.Widget,)


class AbstractButton(QtWidgets.QAbstractButton):
    def serialize_fields(self):
        return dict(
            text=self.text(),
            icon=gui.Icon(self.icon()) if not self.icon().isNull() else None,
            checkable=self.isCheckable(),
            checked=self.isChecked(),
        )

    def __setstate__(self, state):
        self.__init__()
        self.setText(state["text"])
        self.set_id(state.get("object_name", ""))
        self.set_icon(state["icon"])
        self.setEnabled(state.get("enabled", True))
        self.setChecked(state.get("checked", False))
        self.setCheckable(state["checkable"])
        self.setToolTip(state.get("tool_tip", ""))
        self.setStatusTip(state.get("status_tip", ""))

    def set_icon(self, icon: gui.icon.IconType):
        """Set the icon for the button.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(icon)

    def set_style_icon(self, icon: str, size: int = 15):
        if icon not in STANDARD_PIXMAPS:
            raise InvalidParamError(icon, STANDARD_PIXMAPS)
        qicon = self.style().standardIcon(STANDARD_PIXMAPS[icon], None, self)
        self.set_icon(qicon)
        self.setIconSize(core.Size(size, size))

    def set_shortcut(self, shortcut: Union[None, QtGui.QKeySequence, str]):
        if shortcut is None:
            shortcut = ""
        self.setShortcut(shortcut)

    def get_shortcut(self) -> gui.KeySequence:
        return gui.KeySequence(self.shortcut())

    def set_text(self, text: str):
        self.setText(text)

    def set_icon_size(self, size: int):
        self.setIconSize(core.Size(size, size))


if __name__ == "__main__":
    app = widgets.app()
    widget = AbstractButton()
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from typing import Callable, Sequence, Optional, Union, Iterator

from qtpy import QtCore, QtWidgets

from prettyqt import core, widgets
from prettyqt.utils import bidict, InvalidParamError


BUTTONS = bidict(
    cancel=QtWidgets.QDialogButtonBox.Cancel,
    ok=QtWidgets.QDialogButtonBox.Ok,
    save=QtWidgets.QDialogButtonBox.Save,
    open=QtWidgets.QDialogButtonBox.Open,
    close=QtWidgets.QDialogButtonBox.Close,
    discard=QtWidgets.QDialogButtonBox.Discard,
    apply=QtWidgets.QDialogButtonBox.Apply,
    reset=QtWidgets.QDialogButtonBox.Reset,
    restore_defaults=QtWidgets.QDialogButtonBox.RestoreDefaults,
    help=QtWidgets.QDialogButtonBox.Help,
    save_all=QtWidgets.QDialogButtonBox.SaveAll,
    yes=QtWidgets.QDialogButtonBox.Yes,
    yes_to_all=QtWidgets.QDialogButtonBox.YesToAll,
    no=QtWidgets.QDialogButtonBox.No,
    no_to_all=QtWidgets.QDialogButtonBox.NoToAll,
    abort=QtWidgets.QDialogButtonBox.Abort,
    retry=QtWidgets.QDialogButtonBox.Retry,
    ignore=QtWidgets.QDialogButtonBox.Ignore,
)

ROLES = bidict(
    invalid=QtWidgets.QDialogButtonBox.InvalidRole,
    accept=QtWidgets.QDialogButtonBox.AcceptRole,
    reject=QtWidgets.QDialogButtonBox.RejectRole,
    destructive=QtWidgets.QDialogButtonBox.DestructiveRole,
    action=QtWidgets.QDialogButtonBox.ActionRole,
    help=QtWidgets.QDialogButtonBox.HelpRole,
    yes=QtWidgets.QDialogButtonBox.YesRole,
    no=QtWidgets.QDialogButtonBox.NoRole,
    apply=QtWidgets.QDialogButtonBox.ApplyRole,
    reset=QtWidgets.QDialogButtonBox.ResetRole,
)

ORIENTATIONS = bidict(horizontal=QtCore.Qt.Horizontal, vertical=QtCore.Qt.Vertical)


QtWidgets.QDialogButtonBox.__bases__ = (widgets.Widget,)


class DialogButtonBox(QtWidgets.QDialogButtonBox):

    button_clicked = core.Signal(str)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.clicked.connect(self.on_click)

    def __len__(self) -> int:
        return len(self.buttons())

    def __getitem__(self, index: str) -> QtWidgets.QPushButton:
        return self.button(BUTTONS[index])

    def __iter__(self) -> Iterator[QtWidgets.QAbstractButton]:
        return iter(self.buttons())

    def __contains__(self, item):
        return self[item] is not None

    @classmethod
    def create(cls, **kwargs):
        box = cls()
        for k, v in kwargs.items():
            box.add_default_button(k, callback=v)
        return box

    def on_click(self, button):
        self.button_clicked.emit(button.objectName())

    def set_horizontal(self):
        self.setOrientation(QtCore.Qt.Horizontal)

    def set_vertical(self):
        self.setOrientation(QtCore.Qt.Vertical)

    def set_orientation(self, orientation: str):
        """Set the orientation of the button box.

        Allowed values are "horizontal", "vertical"

        Args:
            orientation: orientation for the button box

        Raises:
            InvalidParamError: orientation does not exist
        """
        if orientation not in ORIENTATIONS:
            raise InvalidParamError(orientation, ORIENTATIONS)
        self.setOrientation(ORIENTATIONS[orientation])

    def get_orientation(self) -> str:
        """Return current orientation.

        Possible values: "horizontal", "vertical"

        Returns:
            orientation
        """
        return ORIENTATIONS.inv[self.orientation()]

    def add_default_buttons(self, buttons: Sequence[str]):
        return [self.add_default_button(btn) for btn in buttons]

    def add_default_button(
        self, button: str, callback: Optional[Callable] = None
    ) -> QtWidgets.QPushButton:
        """Add a default button.

        Valid arguments: "cancel", "ok", "save", "open", "close",
                         "discard", "apply", "reset", "restore_defaults",
                         "help", "save_all", "yes", "yes_to_all", "no",
                         "no_to_all", "abort", "retry", "ignore"

        Args:
            button: button to add
            callback: function to call when button gets clicked

        Returns:
            created button

        Raises:
            InvalidParamError: Button type not available
        """
        if button not in BUTTONS:
            raise InvalidParamError(button, BUTTONS)
        btn = self.addButton(BUTTONS[button])
        btn.setObjectName(button)
        if callback:
            btn.clicked.connect(callback)
        return btn

    def add_button(
        self,
        button: Union[QtWidgets.QPushButton, str],
        role: str = "accept",
        callback: Optional[Callable] = None,
    ) -> widgets.PushButton:
        """Add a button.

        Args:
            button: button to add
            role: role of the button
            callback: function to call when button gets clicked

        Returns:
            created button

        Raises:
            InvalidParamError: Button type not available
        """
        if isinstance(button, str):
            button = widgets.PushButton(button)
        self.addButton(button, ROLES[role])
        if callback:
            button.clicked.connect(callback)
        return button


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    widget = DialogButtonBox()
    buttons = list(BUTTONS.keys())
    widget.add_default_buttons(buttons)
    widget.button_clicked.connect(print)
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core


QtCore.QTransposeProxyModel.__bases__ = (core.AbstractProxyModel,)


class TransposeProxyModel(QtCore.QTransposeProxyModel):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt.utils import bidict

SURFACE_CLASS = bidict(window=QtGui.QSurface.Window, offscreen=QtGui.QSurface.Offscreen)

SURFACE_TYPES = bidict(
    raster=QtGui.QSurface.RasterSurface,
    open_gl=QtGui.QSurface.OpenGLSurface,
    raster_gl=QtGui.QSurface.RasterGLSurface,
    open_vg=QtGui.QSurface.OpenVGSurface,
    vulkan=QtGui.QSurface.VulkanSurface,
    metal=QtGui.QSurface.MetalSurface,
)


class Surface(QtGui.QSurface):
    def __repr__(self):
        return f"{self.__class__.__name__}()"

    def get_surface_class(self) -> str:
        """Get the current surface class.

        Possible values: "window", "offscreen"

        Returns:
            surface class
        """
        return SURFACE_CLASS.inv[self.surfaceClass()]

    def get_surface_type(self) -> str:
        """Get the current surface type.

        Possible values: "raster", "open_gl", "raster_gl", "open_vg", "vulkan", "metal"

        Returns:
            surface type
        """
        return SURFACE_TYPES.inv[self.surfaceType()]


if __name__ == "__main__":
    surface = Surface()



from typing import List, Union

from qtpy import QtGui, QtCore

from prettyqt import gui
from prettyqt.utils import bidict, InvalidParamError


IMAGE_WRITER_ERROR = bidict(
    device=QtGui.QImageWriter.DeviceError,
    unsupported_format=QtGui.QImageWriter.UnsupportedFormatError,
    invalid_image=QtGui.QImageWriter.InvalidImageError,
    unknown=QtGui.QImageWriter.UnknownError,
)

TRANSFORMATIONS = gui.imageiohandler.TRANSFORMATIONS


class ImageWriter(QtGui.QImageWriter):
    def __setitem__(self, key: str, val: str):
        self.setText(key, val)

    def get_error(self) -> str:
        """Return error type.

        possible values are "device", "unsupported_format", "invalid_image", "unknown"

        Returns:
            error type
        """
        return IMAGE_WRITER_ERROR.inv[self.error()]

    def get_format(self) -> str:
        return bytes(self.format()).decode()

    def get_subtype(self) -> str:
        return bytes(self.subType()).decode()

    def get_supported_image_formats(self) -> List[str]:
        return [bytes(i).decode() for i in self.supportedImageFormats()]

    def get_supported_subtypes(self) -> List[str]:
        return [bytes(i).decode() for i in self.supportedSubTypes()]

    def set_subtype(self, subtype: Union[str, bytes, QtCore.QByteArray]):
        if isinstance(subtype, str):
            subtype = subtype.encode()
        self.setSubType(subtype)

    def set_format(self, fmt: Union[str, bytes, QtCore.QByteArray]):
        if isinstance(fmt, str):
            fmt = fmt.encode()
        self.setFormat(fmt)

    def set_transformation(self, origin: str):
        """Set the image transformations metadata including orientation.

        Allowed values are "none", "mirror", "flip", "rotate_180", "roate_90",
                           "mirror_and_rotate_90", "flip_and_rotate_90", "rotate_270"

        Args:
            origin: transformation to use

        Raises:
            InvalidParamError: transformation does not exist
        """
        if origin not in TRANSFORMATIONS:
            raise InvalidParamError(origin, TRANSFORMATIONS)
        self.setTransformation(TRANSFORMATIONS[origin])

    def get_transformation(self) -> str:
        """Return the transformation and orientation the image has been set to.

        Possible values: "none", "mirror", "flip", "rotate_180", "roate_90",
                         "mirror_and_rotate_90", "flip_and_rotate_90", "rotate_270"

        Returns:
            transformation
        """
        return TRANSFORMATIONS.inv[self.transformation()]


if __name__ == "__main__":
    writer = ImageWriter()
    print(writer.get_supported_image_formats())
    print(writer.get_subtype())
    print(writer.get_format())
    writer.set_subtype("A8R8G8B8")



# -*- coding: utf-8 -*-

"""Qml module.

Contains QtQml-based classes
"""

from .jsvalue import JSValue
from .jsvalueiterator import JSValueIterator
from .jsengine import JSEngine
from .qmlengine import QmlEngine
from .qmlapplicationengine import QmlApplicationEngine
from .qmlcomponent import QmlComponent

__all__ = [
    "JSValue",
    "JSValueIterator",
    "QmlEngine",
    "JSEngine",
    "QmlApplicationEngine",
    "QmlComponent",
]



# -*- coding: utf-8 -*-

from __future__ import annotations

from qtpy.QtCharts import QtCharts

from prettyqt import charts

QtCharts.QBarCategoryAxis.__bases__ = (charts.AbstractAxis,)


class BarCategoryAxis(QtCharts.QBarCategoryAxis):
    def __delitem__(self, index: str):
        self.remove(index)

    def __getitem__(self, index: int) -> str:
        return self.categories()[index]

    def __setitem__(self, index: str, value: str):
        self.replace(index, value)

    def __add__(self, other: str) -> BarCategoryAxis:
        self.append(other)
        return self



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import gui, widgets

COMPLEX_CONTROLS = widgets.style.COMPLEX_CONTROLS

QtWidgets.QStylePainter.__bases__ = (gui.Painter,)


class StylePainter(QtWidgets.QStylePainter):
    def draw_complex_control(self, control: str, option: QtWidgets.QStyleOptionComplex):
        self.drawComplexControl(COMPLEX_CONTROLS[control], option)



# -*- coding: utf-8 -*-

from qtpy import QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict

QStyleOptionHeader = QtWidgets.QStyleOptionHeader

SECTION_POSITION = bidict(
    beginning=QtWidgets.QStyleOptionHeader.Beginning,
    middle=QtWidgets.QStyleOptionHeader.Middle,
    end=QtWidgets.QStyleOptionHeader.End,
    only_one_section=QtWidgets.QStyleOptionHeader.OnlyOneSection,
)

SELECTED_POSITION = bidict(
    not_adjacent=QStyleOptionHeader.NotAdjacent,
    next_is_selected=QStyleOptionHeader.NextIsSelected,
    previous_is_selected=QStyleOptionHeader.PreviousIsSelected,
    next_and_previous_are_selected=QStyleOptionHeader.NextAndPreviousAreSelected,
)

SORT_INDICATOR = bidict(
    none=QtWidgets.QStyleOptionHeader.SortIndicator(0),
    sort_up=QtWidgets.QStyleOptionHeader.SortUp,
    sort_down=QtWidgets.QStyleOptionHeader.SortDown,
)


QtWidgets.QStyleOptionHeader.__bases__ = (widgets.StyleOption,)


class StyleOptionHeader(QtWidgets.QStyleOptionHeader):
    pass



# -*- coding: utf-8 -*-

"""Gui module.

Contains QtGui-based classes
"""

from qtpy.QtGui import QKeyEvent as KeyEvent
from qtpy.QtGui import QMouseEvent as MouseEvent
from qtpy.QtGui import QWheelEvent as WheelEvent
from qtpy.QtGui import QActionEvent as ActionEvent
from qtpy.QtGui import QShowEvent as ShowEvent
from qtpy.QtGui import QDropEvent as DropEvent
from qtpy.QtGui import QFocusEvent as FocusEvent
from qtpy.QtGui import QDragEnterEvent as DragEnterEvent
from qtpy.QtGui import QContextMenuEvent as ContextMenuEvent
from qtpy.QtGui import QResizeEvent as ResizeEvent
from qtpy.QtGui import QNativeGestureEvent as NativeGestureEvent
from qtpy.QtGui import QMoveEvent as MoveEvent
from qtpy.QtGui import QInputMethodEvent as InputMethodEvent
from qtpy.QtGui import QInputMethodQueryEvent as InputMethodQueryEvent
from qtpy.QtGui import QCloseEvent as CloseEvent
from qtpy.QtGui import QDragLeaveEvent as DragLeaveEvent
from qtpy.QtGui import QHelpEvent as HelpEvent
from qtpy.QtGui import QHideEvent as HideEvent
from qtpy.QtGui import QHoverEvent as HoverEvent
from qtpy.QtGui import QDragMoveEvent as DragMoveEvent
from qtpy.QtGui import QEnterEvent as EnterEvent
from qtpy.QtGui import QExposeEvent as ExposeEvent
from qtpy.QtGui import QFileOpenEvent as FileOpenEvent
from qtpy.QtGui import QIconDragEvent as IconDragEvent
from qtpy.QtGui import QInputEvent as InputEvent
from qtpy.QtGui import QPaintEvent as PaintEvent
from qtpy.QtGui import QShortcutEvent as ShortcutEvent
from qtpy.QtGui import QStatusTipEvent as StatusTipEvent
from qtpy.QtGui import QTouchEvent as TouchEvent
from qtpy.QtGui import QTabletEvent as TabletEvent
from qtpy.QtGui import QWindowStateChangeEvent as WindowStateChangeEvent
from qtpy.QtGui import QWhatsThisClickedEvent as WhatsThisClickedEvent
from qtpy.QtGui import QScrollEvent as ScrollEvent
from qtpy.QtGui import QScrollPrepareEvent as ScrollPrepareEvent
from qtpy.QtGui import QPlatformSurfaceEvent as PlatformSurfaceEvent

from .screen import Screen
from .keysequence import KeySequence
from .surface import Surface
from .window import Window
from .icon import Icon
from .paintdevice import PaintDevice
from .paintdevicewindow import PaintDeviceWindow
from .pixmap import Pixmap
from .pixmapcache import PixmapCache
from .bitmap import Bitmap
from .image import Image
from .openglwindow import OpenGLWindow
from .rasterwindow import RasterWindow
from .clipboard import Clipboard
from .inputmethod import InputMethod
from .sessionmanager import SessionManager
from .guiapplication import GuiApplication
from .validator import Validator
from .regexpvalidator import RegExpValidator
from .regularexpressionvalidator import RegularExpressionValidator
from .textoption import TextOption
from .textblock import TextBlock
from .textdocument import TextDocument
from .statictext import StaticText
from .intvalidator import IntValidator
from .doublevalidator import DoubleValidator
from .brush import Brush
from .textblockuserdata import TextBlockUserData
from .color import Color
from .gradient import Gradient
from .lineargradient import LinearGradient
from .radialgradient import RadialGradient
from .conicalgradient import ConicalGradient
from .pagesize import PageSize
from .pagelayout import PageLayout
from .font import Font
from .fontmetrics import FontMetrics
from .fontmetricsf import FontMetricsF
from .fontinfo import FontInfo
from .fontdatabase import FontDatabase
from .region import Region
from .movie import Movie
from .transform import Transform
from .pagedpaintdevice import PagedPaintDevice
from .pen import Pen
from .picture import Picture
from .painter import Painter
from .painterpath import PainterPath
from .painterpathstroker import PainterPathStroker
from .palette import Palette
from .cursor import Cursor
from .polygon import Polygon
from .polygonf import PolygonF
from .standarditem import StandardItem
from .standarditemmodel import StandardItemModel
from .textcursor import TextCursor
from .syntaxhighlighter import SyntaxHighlighter
from .pdfwriter import PdfWriter
from .desktopservices import DesktopServices
from .matrix4x4 import Matrix4x4
from .vector3d import Vector3D
from .vector4d import Vector4D
from .imageiohandler import ImageIOHandler
from .imagereader import ImageReader
from .imagewriter import ImageWriter
from .textformat import TextFormat
from .textlength import TextLength
from .textframeformat import TextFrameFormat
from .textcharformat import TextCharFormat
from .textimageformat import TextImageFormat
from .textlistformat import TextListFormat
from .texttablecellformat import TextTableCellFormat
from .textobject import TextObject
from .textblockgroup import TextBlockGroup
from .textframe import TextFrame
from .abstracttextdocumentlayout import AbstractTextDocumentLayout

from prettyqt import core

if core.VersionNumber.get_qt_version() >= (5, 13, 0):
    from .colorspace import ColorSpace


def app():
    if GuiApplication.instance() is not None:
        return GuiApplication.instance()
    return GuiApplication([])


__all__ = [
    "app",
    "KeyEvent",
    "MouseEvent",
    "WheelEvent",
    "ActionEvent",
    "ShowEvent",
    "ContextMenuEvent",
    "ResizeEvent",
    "NativeGestureEvent",
    "InputMethodQueryEvent",
    "InputMethodEvent",
    "SessionManager",
    "CloseEvent",
    "DragLeaveEvent",
    "MoveEvent",
    "HelpEvent",
    "HideEvent",
    "HoverEvent",
    "DragMoveEvent",
    "TouchEvent",
    "TabletEvent",
    "WindowStateChangeEvent",
    "FileOpenEvent",
    "IconDragEvent",
    "InputEvent",
    "PaintEvent",
    "ShortcutEvent",
    "StatusTipEvent",
    "EnterEvent",
    "ExposeEvent",
    "WhatsThisClickedEvent",
    "ScrollEvent",
    "ScrollPrepareEvent",
    "PlatformSurfaceEvent",
    "DropEvent",
    "DragEnterEvent",
    "FocusEvent",
    "Screen",
    "Gradient",
    "LinearGradient",
    "RadialGradient",
    "ConicalGradient",
    "PageSize",
    "PageLayout",
    "Clipboard",
    "GuiApplication",
    "Validator",
    "RegExpValidator",
    "TextOption",
    "TextBlock",
    "TextBlockGroup",
    "TextDocument",
    "StaticText",
    "RegularExpressionValidator",
    "IntValidator",
    "DoubleValidator",
    "Brush",
    "TextBlockUserData",
    "Color",
    "Font",
    "FontMetrics",
    "FontMetricsF",
    "FontInfo",
    "FontDatabase",
    "Region",
    "Movie",
    "PaintDevice",
    "Transform",
    "PagedPaintDevice",
    "Icon",
    "Image",
    "Pen",
    "Picture",
    "Pixmap",
    "PixmapCache",
    "Bitmap",
    "Painter",
    "PainterPath",
    "PainterPathStroker",
    "Palette",
    "Cursor",
    "Polygon",
    "PolygonF",
    "StandardItem",
    "StandardItemModel",
    "TextCharFormat",
    "TextImageFormat",
    "TextListFormat",
    "TextTableCellFormat",
    "TextCursor",
    "SyntaxHighlighter",
    "PdfWriter",
    "KeySequence",
    "Surface",
    "Window",
    "DesktopServices",
    "Matrix4x4",
    "Vector3D",
    "Vector4D",
    "PaintDeviceWindow",
    "OpenGLWindow",
    "RasterWindow",
    "ImageIOHandler",
    "ImageReader",
    "ImageWriter",
    "TextObject",
    "TextLength",
    "TextFormat",
    "TextFrameFormat",
    "TextFrame",
    "AbstractTextDocumentLayout",
    "InputMethod",
    "ColorSpace",
]



# -*- coding: utf-8 -*-

"""Multimediawidgets module."""

from prettyqt import core

if core.VersionNumber.get_qt_version() >= (5, 13, 0):
    from .videowidgetcontrol import VideoWidgetControl
from .videowidget import VideoWidget
from .graphicsvideoitem import GraphicsVideoItem

__all__ = ["VideoWidgetControl", "VideoWidget", "GraphicsVideoItem"]



# -*- coding: utf-8 -*-

from typing import Union

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning


from qtpy import QtCore

from prettyqt import positioning
from prettyqt.utils import bidict

QNmeaPositionInfoSource = QtPositioning.QNmeaPositionInfoSource

UPDATE_MODES = bidict(
    real_time=QtPositioning.QNmeaPositionInfoSource.RealTimeMode,
    simulation=QtPositioning.QNmeaPositionInfoSource.SimulationMode,
)

QtPositioning.QNmeaPositionInfoSource.__bases__ = (positioning.GeoPositionInfoSource,)


class NmeaPositionInfoSource(QtPositioning.QNmeaPositionInfoSource):
    def __init__(self, update_mode: Union[int, str], parent: QtCore.QObject):
        if update_mode in UPDATE_MODES:
            update_mode = UPDATE_MODES[update_mode]
        super().__init__(update_mode, parent)

    def __repr__(self):
        return "NmeaPositionInfoSource()"

    def get_update_mode(self) -> str:
        return UPDATE_MODES.inv[self.updateMode()]


if __name__ == "__main__":
    obj = QtCore.QObject()
    source = NmeaPositionInfoSource("real_time", obj)
    print(str(source))
    print(repr(source))



# -*- coding: utf-8 -*-

from __future__ import annotations

from qtpy import QtGui

from prettyqt import core


QtGui.QValidator.__bases__ = (core.Object,)


class Validator(QtGui.QValidator):
    def __repr__(self):
        return f"{self.__class__.__name__}()"

    def __add__(self, other: Validator):
        if isinstance(other, Validator):
            from prettyqt import custom_validators

            return custom_validators.CompositeValidator([self, other])

    def __radd__(self, other: QtGui.QValidator):
        """Needed for sum()."""
        return self.__add__(other)

    def is_valid_value(self, value: str, pos: int = 0) -> bool:
        val = self.validate(value, pos)
        return val[0] == self.Acceptable


if __name__ == "__main__":
    val = Validator()



from qtpy import QtWidgets, QtCore

from prettyqt import widgets
from prettyqt.utils import bidict

ANCHORS = bidict(
    left=QtCore.Qt.AnchorLeft,
    horizontal_center=QtCore.Qt.AnchorHorizontalCenter,
    right=QtCore.Qt.AnchorRight,
    top=QtCore.Qt.AnchorTop,
    vertical_center=QtCore.Qt.AnchorVerticalCenter,
    bottom=QtCore.Qt.AnchorBottom,
)

CORNERS = bidict(
    top_left=QtCore.Qt.TopLeftCorner,
    top_right=QtCore.Qt.TopRightCorner,
    bottom_left=QtCore.Qt.BottomLeftCorner,
    bottom_right=QtCore.Qt.BottomRightCorner,
)

QtWidgets.QGraphicsAnchorLayout.__bases__ = (widgets.GraphicsLayout,)


class GraphicsAnchorLayout(QtWidgets.QGraphicsAnchorLayout):
    def add_anchor(
        self,
        first_item: QtWidgets.QGraphicsLayoutItem,
        first_edge: str,
        second_item: QtWidgets.QGraphicsLayoutItem,
        second_edge: str,
    ):
        self.addAnchor(first_item, ANCHORS[first_edge], second_item, ANCHORS[second_edge])

    def get_anchor(
        self,
        first_item: QtWidgets.QGraphicsLayoutItem,
        first_edge: str,
        second_item: QtWidgets.QGraphicsLayoutItem,
        second_edge: str,
    ):
        return self.anchor(
            first_item, ANCHORS[first_edge], second_item, ANCHORS[second_edge]
        )

    def add_anchors(
        self,
        first_item: QtWidgets.QGraphicsLayoutItem,
        second_item: QtWidgets.QGraphicsLayoutItem,
        orientation: str,
    ):
        if orientation == "horizontal":
            orientation = QtCore.Qt.Horizontal
        elif orientation == "vertical":
            orientation = QtCore.Qt.Vertical
        else:
            raise ValueError()
        self.addAnchors(first_item, second_item, orientation)

    def add_corner_anchors(
        self,
        first_item: QtWidgets.QGraphicsLayoutItem,
        first_corner: str,
        second_item: QtWidgets.QGraphicsLayoutItem,
        second_corner: str,
    ):
        self.addCornerAnchors(
            first_item, CORNERS[first_corner], second_item, CORNERS[second_corner]
        )


if __name__ == "__main__":
    app = widgets.app()
    layout = GraphicsAnchorLayout()



# -*- coding: utf-8 -*-

from typing import Union, Optional

from qtpy import QtWidgets

from prettyqt import core, widgets


class StringOrNumberWidget(widgets.GroupBox):

    value_changed = core.Signal(object)

    def __init__(self, title: str = "", parent: Optional[QtWidgets.QWidget] = None):
        super().__init__(checkable=False, title=title)
        self.set_layout("vertical")
        self.rb_lineedit = widgets.RadioButton("String")
        self.lineedit = widgets.LineEdit()
        self.rb_spinbox = widgets.RadioButton("Number")
        self.spinbox = widgets.DoubleSpinBox()
        layout_lineedit = widgets.BoxLayout("horizontal")
        layout_lineedit.add(self.rb_lineedit)
        layout_lineedit.add(self.lineedit)
        layout_spinbox = widgets.BoxLayout("horizontal")
        layout_spinbox.add(self.rb_spinbox)
        layout_spinbox.add(self.spinbox)
        self.box.add(layout_lineedit)
        self.box.add(layout_spinbox)
        self.rb_spinbox.toggled.connect(self.spinbox.setEnabled)
        self.rb_spinbox.toggled.connect(self.lineedit.setDisabled)
        self.rb_lineedit.toggled.connect(self.lineedit.setEnabled)
        self.rb_lineedit.toggled.connect(self.spinbox.setDisabled)
        self.spinbox.value_changed.connect(self.on_value_change)
        self.lineedit.value_changed.connect(self.on_value_change)
        self.rb_lineedit.setChecked(True)

    def on_value_change(self):
        value = self.get_value()
        self.value_changed.emit(value)

    def get_value(self) -> Union[float, str]:
        if self.rb_spinbox.isChecked():
            val = self.spinbox.get_value()
            return int(val) if val.is_integer() else val
        else:
            return self.lineedit.get_value()

    def set_value(self, value: Union[float, str]):
        if isinstance(value, str):
            self.rb_lineedit.setChecked(True)
            self.lineedit.set_value(value)
        elif isinstance(value, (int, float)):
            self.rb_spinbox.setChecked(True)
            self.spinbox.set_value(value)
        else:
            raise TypeError(f"Invalid Type for set_value: {type(value)}")


if __name__ == "__main__":
    app = widgets.app()
    widget = StringOrNumberWidget("Test")
    widget.value_changed.connect(print)
    widget.show()
    app.main_loop()
    print(widget.enabled)



# -*- coding: utf-8 -*-

from typing import Callable, Optional, Dict, Any

from qtpy import QtCore, QtWidgets

from prettyqt import core, gui, widgets
from prettyqt.utils import bidict, InvalidParamError, helpers


STYLES = bidict(
    icon=QtCore.Qt.ToolButtonIconOnly,
    text=QtCore.Qt.ToolButtonTextOnly,
    text_beside_icon=QtCore.Qt.ToolButtonTextBesideIcon,
    text_below_icon=QtCore.Qt.ToolButtonTextUnderIcon,
)

TOOLBAR_AREAS = bidict(
    left=QtCore.Qt.LeftToolBarArea,
    right=QtCore.Qt.RightToolBarArea,
    top=QtCore.Qt.TopToolBarArea,
    bottom=QtCore.Qt.BottomToolBarArea,
    all=QtCore.Qt.AllToolBarAreas,
    none=QtCore.Qt.NoToolBarArea,
)


QtWidgets.QToolBar.__bases__ = (widgets.Widget,)


class ToolBar(QtWidgets.QToolBar):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_icon_size(24)
        self.menu_buttons = list()

    def __setstate__(self, state: Dict[str, Any]) -> None:
        super().__init__()
        self.addActions(state["actions"])

    def add_menu_button(
        self, label: str, icon: gui.icon.IconType, menu: QtWidgets.QMenu
    ) -> widgets.ToolButton:
        btn = widgets.ToolButton.for_menu(menu)
        btn.setText(label)
        btn.setToolButtonStyle(self.toolButtonStyle())
        btn.set_icon(icon)
        self.menu_buttons.append(btn)
        self.addWidget(btn)
        return btn

    def add_separator(
        self, text: Optional[str] = None, before: QtWidgets.QAction = None
    ) -> QtWidgets.QAction:
        """Adds a separator showing an optional label.

        Args:
            text: Text to show on separator
            before: insert separator before specific action

        Returns:
            Separator action
        """
        if text is None:
            if before:
                return self.insertSeparator(before)
            else:
                return self.addSeparator()
        else:
            label = widgets.Label(text)
            label.setMinimumWidth(self.minimumWidth())
            with label.edit_stylesheet() as ss:
                ss.background.setValue("lightgrey")
            label.set_alignment(horizontal="center")
            if before:
                return self.insertWidget(before, label)
            else:
                return self.addWidget(label)

    def set_style(self, style: str):
        if style is None:
            return None
        self.setToolButtonStyle(STYLES[style])
        for btn in self.menu_buttons:
            btn.set_style(style)

    def get_style(self) -> str:
        """Return current style.

        Possible values: "icon", "text", "text_below_icon", "text_beside_icon"

        Returns:
            style
        """
        return STYLES.inv[self.toolButtonStyle()]

    def add_action(
        self,
        label: str,
        icon: gui.icon.IconType = None,
        callback: Optional[Callable] = None,
        checkable: bool = False,
    ) -> QtWidgets.QAction:
        icon = gui.icon.get_icon(icon)
        action = self.addAction(icon, label)
        if callback is not None:
            action.triggered.connect(callback)
        if checkable:
            action.setCheckable(True)
        return action

    def add_spacer(self) -> QtWidgets.QAction:
        spacer = widgets.Widget()
        spacer.set_size_policy("expanding", "expanding")
        return self.addWidget(spacer)

    def set_icon_size(self, size: int):
        self.setIconSize(core.Size(size, size))

    def set_font_size(self, size: int):
        with self.current_font() as font:
            font.set_size(size)

    def is_area_allowed(self, area: str) -> bool:
        """Check if toolbar is allowed at specified area.

        Valid values for area: "left", "right", "top", "bottom", "all"

        Args:
            area: area of the toolbar

        Raises:
            InvalidParamError: area does not exist
        """
        if area not in TOOLBAR_AREAS:
            raise InvalidParamError(area, TOOLBAR_AREAS)
        return self.isAreaAllowed(TOOLBAR_AREAS[area])

    def set_allowed_areas(self, *areas: str):
        for area in areas:
            if area not in TOOLBAR_AREAS:
                raise InvalidParamError(area, TOOLBAR_AREAS)
        flag = helpers.merge_flags(areas, TOOLBAR_AREAS)
        self.setAllowedAreas(flag)


if __name__ == "__main__":
    app = widgets.app()
    widget = ToolBar()
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-

"""Network module.

Contains QtNetWork-based classes
"""

from .networkcookie import NetworkCookie
from .networkcookiejar import NetworkCookieJar
from .networkrequest import NetworkRequest
from .networkproxy import NetworkProxy
from .hostaddress import HostAddress
from .networkaddressentry import NetworkAddressEntry
from .networkinterface import NetworkInterface
from .networkdatagram import NetworkDatagram
from .abstractsocket import AbstractSocket
from .localserver import LocalServer
from .tcpserver import TcpServer
from .tcpsocket import TcpSocket
from .udpsocket import UdpSocket
from .httppart import HttpPart
from .httpmultipart import HttpMultiPart
from .networkaccessmanager import NetworkAccessManager

__all__ = [
    "NetworkProxy",
    "AbstractSocket",
    "HostAddress",
    "LocalServer",
    "TcpServer",
    "NetworkDatagram",
    "NetworkAddressEntry",
    "NetworkInterface",
    "TcpSocket",
    "UdpSocket",
    "HttpPart",
    "HttpMultiPart",
    "NetworkCookie",
    "NetworkCookieJar",
    "NetworkRequest",
    "NetworkAccessManager",
]



# -*- coding: utf-8 -*-

from qtpy import QtGui


class TextBlock(QtGui.QTextBlock):
    def __repr__(self):
        return f"TextBlock({self.text()!r})"

    def __contains__(self, position: int):
        return self.contains(position)


if __name__ == "__main__":
    doc = TextBlock()



# -*- coding: utf-8 -*-
from typing import Union

from qtpy import QtNetwork, QtCore

from prettyqt import core, network
from prettyqt.utils import bidict, InvalidParamError

OPERATIONS = bidict(
    head=QtNetwork.QNetworkAccessManager.HeadOperation,
    get=QtNetwork.QNetworkAccessManager.GetOperation,
    put=QtNetwork.QNetworkAccessManager.PutOperation,
    post=QtNetwork.QNetworkAccessManager.PostOperation,
    delete=QtNetwork.QNetworkAccessManager.DeleteOperation,
    custom=QtNetwork.QNetworkAccessManager.CustomOperation,
)

REDIRECT_POLICIES = network.networkrequest.REDIRECT_POLICIES

QtNetwork.QNetworkAccessManager.__bases__ = (core.Object,)


class NetworkAccessManager(QtNetwork.QNetworkAccessManager):
    # def request(
    #     self,
    #     method,
    #     url,
    #     headers=None,
    #     cookies=None,
    #     files=None,
    #     auth=None,
    #     timeout=None,
    #     allow_redirects=True,
    # ):
    #     req = network.NetworkRequest()
    #     if allow_redirects:
    #         pass

    def get(self, request: Union[str, QtCore.QUrl, QtNetwork.QNetworkRequest]):
        if isinstance(request, str):
            request = core.Url(request)
        request = network.NetworkRequest(request)
        return super().get(request)

    # def post(self, url, data=None, json=None):
    #     pass

    # def put(self, url, data=None, json=None):
    #     pass

    # def patch(self, url, data=None):
    #     pass

    # def delete(self, url):
    #     pass

    def set_redirect_policy(self, policy: str):
        """Set redirect policy.

        Valid values: "manual", "no_less_safe", "same_origin", "user_verified"

        Args:
            policy: redirect policy

        Raises:
            InvalidParamError: redirect policy does not exist
        """
        if policy not in REDIRECT_POLICIES:
            raise InvalidParamError(policy, REDIRECT_POLICIES)
        self.setRedirectPolicy(REDIRECT_POLICIES[policy])

    def get_redirect_policy(self) -> str:
        """Get the current redirect policy.

        Possible values: "manual", "no_less_safe", "same_origin", "user_verified"

        Returns:
            redirect policy
        """
        return REDIRECT_POLICIES.inv[self.redirectPolicy()]
